"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Utilities
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.UiCore = void 0;
/** Import color themes, layout variables and Sass classes barrel file */
require("./colorthemes.scss");
require("./layout-variables.scss");
require("./classes.scss");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_abstract_1 = require("@bentley/ui-abstract");
/**
 * Manages the I18N service for the ui-core package.
 * @public
 */
class UiCore {
    /**
     * Registers the I18N service namespace for UiCore. Also initializes UiAbstract.
     * @param i18n The internationalization service created by the application.
     */
    static async initialize(i18n) {
        if (UiCore._initialized) {
            bentleyjs_core_1.Logger.logInfo(UiCore.loggerCategory(UiCore), `UiCore.initialize already called`);
            return;
        }
        UiCore._i18n = i18n;
        await UiCore._i18n.registerNamespace(UiCore.i18nNamespace).readFinished;
        await ui_abstract_1.UiAbstract.initialize(i18n);
        UiCore._initialized = true;
    }
    /** Unregisters the UiCore I18N namespace */
    static terminate() {
        if (UiCore._i18n)
            UiCore._i18n.unregisterNamespace(UiCore.i18nNamespace);
        UiCore._i18n = undefined;
        ui_abstract_1.UiAbstract.terminate();
        UiCore._initialized = false;
    }
    /** Determines if UiCore has been initialized */
    static get initialized() { return UiCore._initialized; }
    /** The internationalization service created by the application. */
    static get i18n() {
        if (!UiCore._i18n)
            throw new ui_abstract_1.UiError(UiCore.loggerCategory(this), "i18n: UiCore.initialize has not been called. Unable to return I18N object.");
        return UiCore._i18n;
    }
    /** The internationalization service namespace. */
    static get i18nNamespace() {
        return "UiCore";
    }
    /** Calls i18n.translateWithNamespace with the "UiCore" namespace. Do NOT include the namespace in the key.
     * @internal
     */
    static translate(key) {
        if (!UiCore.initialized) {
            bentleyjs_core_1.Logger.logError(UiCore.loggerCategory(this), `translate: UiCore.initialize has not been called. Returning blank string.`);
            return "";
        }
        return UiCore.i18n.translateWithNamespace(UiCore.i18nNamespace, key);
    }
    /** @internal */
    static get packageName() {
        return "ui-core";
    }
    /** @internal */
    static loggerCategory(obj) {
        const className = ui_abstract_1.getClassName(obj);
        const category = UiCore.packageName + (className ? `.${className}` : "");
        return category;
    }
}
exports.UiCore = UiCore;
UiCore._initialized = false;
//# sourceMappingURL=UiCore.js.map