"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module AutoSuggest
 */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutoSuggest = void 0;
require("./AutoSuggest.scss");
const React = require("react");
const ReactAutosuggest = require("react-autosuggest");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_abstract_1 = require("@bentley/ui-abstract");
const UiCore_1 = require("../UiCore");
/** Auto Suggest React component. Uses the react-autosuggest component internally.
 * @beta
 */
class AutoSuggest extends React.PureComponent {
    constructor(props) {
        super(props);
        this._isMounted = false;
        this._onChange = (e) => {
            let newValue = "";
            if (e.target.tagName === "LI" && e.target.textContent)
                newValue = e.target.textContent;
            else if (e.target.value)
                newValue = e.target.value;
            this.setState({
                inputValue: newValue,
            });
        };
        this._onFocus = (e) => {
            // istanbul ignore else
            if (this.props.onInputFocus)
                this.props.onInputFocus(e);
        };
        /** Autosuggest will call this function every time you need to update suggestions. */
        this._onSuggestionsFetchRequested = async (request) => {
            const value = request.value;
            const suggestions = await this._getSuggestions(value);
            if (this._isMounted)
                this.setState({ suggestions });
        };
        /** Autosuggest will call this function every time you need to clear suggestions. */
        this._onSuggestionsClearRequested = () => {
            this.setState({ suggestions: [] });
            this.props.onSuggestionsClearRequested && this.props.onSuggestionsClearRequested();
        };
        this._onSuggestionSelected = (_event, data) => {
            this.props.onSuggestionSelected(data.suggestion);
        };
        /** Teach Autosuggest how to calculate suggestions for any given input value. */
        this._getSuggestions = async (value) => {
            if (typeof this.props.options === "function")
                return Promise.resolve(this.props.options(value));
            if (this.props.getSuggestions)
                return this.props.getSuggestions(value);
            if (this.props.options === undefined) {
                bentleyjs_core_1.Logger.logError(UiCore_1.UiCore.loggerCategory(this), `props.options or props.getSuggestions should be provided`);
                return Promise.resolve([]);
            }
            const inputValue = value.trim().toLowerCase();
            const inputLength = inputValue.length;
            return Promise.resolve(inputLength === 0 ?
                /* istanbul ignore next */ [] :
                this.props.options.filter((data) => {
                    return data.label.toLowerCase().includes(inputValue) || data.value.toLowerCase().includes(inputValue);
                }));
        };
        /** When suggestion is clicked, Autosuggest needs to populate the input based on the clicked suggestion.  */
        this._getSuggestionValue = (suggestion) => suggestion.label;
        /** Render each suggestion. */
        this._renderSuggestion = (suggestion) => (React.createElement("span", null, suggestion.label));
        this._handleKeyDown = (e) => {
            switch (e.key) {
                case ui_abstract_1.SpecialKey.Enter:
                    // istanbul ignore else
                    if (this.props.onPressEnter)
                        this.props.onPressEnter(e);
                    break;
                case ui_abstract_1.SpecialKey.Escape:
                    // istanbul ignore else
                    if (this.props.onPressEscape)
                        this.props.onPressEscape(e);
                    break;
                case ui_abstract_1.SpecialKey.Tab:
                    // istanbul ignore else
                    if (this.props.onPressTab)
                        this.props.onPressTab(e);
                    break;
            }
        };
        this._theme = {
            container: "uicore-autosuggest__container",
            containerOpen: "uicore-autosuggest__container--open",
            input: "uicore-autosuggest__input",
            inputOpen: "uicore-autosuggest__input--open",
            inputFocused: "uicore-autosuggest__input--focused",
            suggestionsContainer: "uicore-autosuggest__suggestions-container",
            suggestionsContainerOpen: "uicore-autosuggest__suggestions-container--open",
            suggestionsList: "uicore-autosuggest__suggestions-list",
            suggestion: "uicore-autosuggest__suggestion",
            suggestionFirst: "uicore-autosuggest__suggestion--first",
            suggestionHighlighted: "uicore-autosuggest__suggestion--highlighted",
            sectionContainer: "uicore-autosuggest__section-container",
            sectionContainerFirst: "uicore-autosuggest__section-container--first",
            sectionTitle: "uicore-autosuggest__section-title",
        };
        // Autosuggest is a controlled component.
        // This means that you need to provide an input value
        // and an onChange handler that updates this value (see below).
        // Suggestions also need to be provided to the Autosuggest,
        // and they are initially empty because the Autosuggest is closed.
        this.state = {
            inputValue: this.getLabel(props.value),
            suggestions: [],
        };
    }
    /** @internal */
    componentDidMount() {
        this._isMounted = true;
    }
    /** @internal */
    componentWillUnmount() {
        this._isMounted = false;
    }
    componentDidUpdate(prevProps) {
        if (this.props.value !== prevProps.value || this.props.options !== prevProps.options) {
            this.setState((_prevState, props) => ({
                inputValue: this.getLabel(props.value),
            }));
        }
    }
    getLabel(value) {
        let label = "";
        if (this.props.getLabel) {
            label = this.props.getLabel(value);
        }
        else if (this.props.options instanceof Array) {
            const entry = this.props.options.find((data) => data.value === value);
            if (entry)
                label = entry.label;
        }
        else {
            bentleyjs_core_1.Logger.logError(UiCore_1.UiCore.loggerCategory(this), `props.getLabel should be provided when props.options is a function`);
        }
        return label;
    }
    render() {
        const { inputValue, suggestions } = this.state;
        const _a = this.props, { value, onChange, placeholder, options, onSuggestionSelected, setFocus, alwaysRenderSuggestions, // eslint-disable-line @typescript-eslint/no-unused-vars
        onPressEnter, onPressEscape, onPressTab, onInputFocus, getLabel, // eslint-disable-line @typescript-eslint/no-unused-vars
        getSuggestions, // eslint-disable-line deprecation/deprecation, @typescript-eslint/no-unused-vars
        renderInputComponent, renderSuggestionsContainer, onSuggestionsClearRequested } = _a, // eslint-disable-line @typescript-eslint/no-unused-vars
        props = __rest(_a, ["value", "onChange", "placeholder", "options", "onSuggestionSelected", "setFocus", "alwaysRenderSuggestions", "onPressEnter", "onPressEscape", "onPressTab", "onInputFocus", "getLabel", "getSuggestions", "renderInputComponent", "renderSuggestionsContainer", "onSuggestionsClearRequested"]);
        const inputPlaceholder = (!inputValue) ? placeholder : undefined;
        const inputProps = Object.assign(Object.assign({}, props), { value: inputValue, onChange: this._onChange, onFocus: this._onFocus, placeholder: inputPlaceholder, autoFocus: setFocus });
        return (
        // The onKeyDown event handler is only being used to capture bubbled events
        // eslint-disable-next-line jsx-a11y/no-static-element-interactions
        React.createElement("div", { className: this.props.className, style: this.props.style, onKeyDown: this._handleKeyDown },
            React.createElement(ReactAutosuggest, { theme: this._theme, suggestions: suggestions, onSuggestionsFetchRequested: this._onSuggestionsFetchRequested, onSuggestionsClearRequested: this._onSuggestionsClearRequested, getSuggestionValue: this._getSuggestionValue, renderSuggestion: this._renderSuggestion, inputProps: inputProps, onSuggestionSelected: this._onSuggestionSelected, alwaysRenderSuggestions: alwaysRenderSuggestions, renderInputComponent: renderInputComponent, renderSuggestionsContainer: renderSuggestionsContainer })));
    }
}
exports.AutoSuggest = AutoSuggest;
//# sourceMappingURL=AutoSuggest.js.map