"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module ContextMenu
 */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContextMenuItem = void 0;
const React = require("react");
const classnames = require("classnames");
const ui_abstract_1 = require("@bentley/ui-abstract");
const BadgeUtilities_1 = require("../badge/BadgeUtilities");
const TildeFinder_1 = require("./TildeFinder");
const IconComponent_1 = require("../icons/IconComponent");
/**
 * Menu Item class for use within a [[ContextMenu]] component.
 * @public
 */
class ContextMenuItem extends React.PureComponent {
    constructor(props) {
        super(props);
        this._root = null;
        /** @internal */
        this.state = {};
        this._updateHotkey = (node) => {
            let hotKey;
            const isDisabled = ui_abstract_1.ConditionalBooleanValue.getValue(this.props.disabled);
            const isHidden = ui_abstract_1.ConditionalBooleanValue.getValue(this.props.hidden);
            if (!isDisabled && !isHidden)
                hotKey = TildeFinder_1.TildeFinder.findAfterTilde(node).character;
            else
                hotKey = undefined;
            if (hotKey && hotKey !== this.state.hotKey) {
                this.setState({ hotKey });
                if (this.props.onHotKeyParsed)
                    this.props.onHotKeyParsed(hotKey);
            }
        };
        this._handleFocus = (event) => {
            event.stopPropagation();
        };
        this._handleMouseOver = (_event) => {
            // istanbul ignore else
            if (this._root && this._root.style.visibility !== "hidden" && this.props.onHover) {
                this.props.onHover();
            }
        };
        this.select = () => {
            // istanbul ignore else
            if (this._root) {
                this._root.click();
                if (this.props.parentMenu && this.props.parentMenu.props.parentSubmenu)
                    this.props.parentMenu.props.parentSubmenu.close(true);
            }
        };
        this._handleClick = (event) => {
            const isDisabled = ui_abstract_1.ConditionalBooleanValue.getValue(this.props.disabled);
            if (isDisabled)
                return;
            if (this.props.onClick)
                this.props.onClick(event);
            if (this.props.onSelect)
                this.props.onSelect(event);
        };
        this._handleKeyUp = (event) => {
            const isDisabled = ui_abstract_1.ConditionalBooleanValue.getValue(this.props.disabled);
            if (event.key === ui_abstract_1.SpecialKey.Enter && this.props.onSelect !== undefined && !isDisabled) {
                this.props.onSelect(event);
            }
        };
    }
    render() {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const _a = this.props, { onClick, className, style, onSelect, icon, disabled, hidden, onHover, isSelected, parentMenu, onHotKeyParsed, badgeType, iconRight, hideIconContainer } = _a, props = __rest(_a, ["onClick", "className", "style", "onSelect", "icon", "disabled", "hidden", "onHover", "isSelected", "parentMenu", "onHotKeyParsed", "badgeType", "iconRight", "hideIconContainer"]);
        const badge = BadgeUtilities_1.BadgeUtilities.getComponentForBadgeType(badgeType);
        const isDisabled = ui_abstract_1.ConditionalBooleanValue.getValue(disabled);
        const isHidden = ui_abstract_1.ConditionalBooleanValue.getValue(hidden);
        if (this._lastChildren !== this.props.children) {
            this._parsedChildren = TildeFinder_1.TildeFinder.findAfterTilde(this.props.children).node;
            this._lastChildren = this.props.children;
        }
        return (React.createElement("div", Object.assign({}, props, { ref: (el) => { this._root = el; }, onClick: this._handleClick, style: style, onFocus: this._handleFocus, onKeyUp: this._handleKeyUp, onMouseOver: this._handleMouseOver, "data-testid": "core-context-menu-item", className: classnames("core-context-menu-item", className, isDisabled && "core-context-menu-disabled", isHidden && "core-context-menu-hidden", isSelected && "core-context-menu-is-selected"), role: "menuitem", tabIndex: isSelected ? 0 : -1, "aria-disabled": isDisabled, "aria-hidden": isHidden }),
            !hideIconContainer && React.createElement("div", { className: "core-context-menu-icon" }, icon !== undefined && React.createElement(IconComponent_1.Icon, { iconSpec: icon })),
            React.createElement("div", { className: "core-context-menu-content" }, this._parsedChildren),
            iconRight &&
                React.createElement("div", { className: classnames("core-context-menu-icon", "core-context-menu-icon-right") },
                    React.createElement(IconComponent_1.Icon, { iconSpec: iconRight })),
            badge &&
                React.createElement("div", { className: "core-context-menu-badge" }, badge)));
    }
    componentDidMount() {
        this._updateHotkey(this.props.children);
    }
    componentDidUpdate(prevProps) {
        if (this.props.children !== prevProps.children) {
            this._updateHotkey(this.props.children);
        }
    }
}
exports.ContextMenuItem = ContextMenuItem;
/** @internal */
ContextMenuItem.defaultProps = {
    disabled: false,
    hidden: false,
    isSelected: false,
};
//# sourceMappingURL=ContextMenuItem.js.map