"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Cube
 */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CubeFace = exports.Cube = exports.Face = void 0;
require("./Cube.scss");
const classnames = require("classnames");
const React = require("react");
const geometry_core_1 = require("@bentley/geometry-core");
/** Cube Face enumeration
 * @public
 */
var Face;
(function (Face) {
    Face["None"] = "";
    Face["Left"] = "left";
    Face["Right"] = "right";
    Face["Back"] = "back";
    Face["Front"] = "front";
    Face["Bottom"] = "bottom";
    Face["Top"] = "top";
})(Face = exports.Face || (exports.Face = {}));
/** Cube React component used by the 3d Cube Navigation Aid
 * @public
 */
class Cube extends React.PureComponent {
    render() {
        const _a = this.props, { faces, rotMatrix, className } = _a, props = __rest(_a, ["faces", "rotMatrix", "className"]);
        return (React.createElement("div", Object.assign({ className: classnames("core-cube-css3d", className), "data-testid": "core-cube" }, props), [Face.Front, Face.Back, Face.Right, Face.Left, Face.Top, Face.Bottom]
            .map((face) => {
            const content = faces && faces[face];
            return (React.createElement(CubeFace, { key: face, rotMatrix: rotMatrix, face: face }, content));
        })));
    }
}
exports.Cube = Cube;
/** @internal */
class CubeFace extends React.Component {
    constructor() {
        super(...arguments);
        this._faceWidth = 0;
    }
    render() {
        const _a = this.props, { rotMatrix, face, style, children } = _a, props = __rest(_a, ["rotMatrix", "face", "style", "children"]);
        if (face === Face.None)
            return null;
        const classes = classnames("face", this.getCSSClassNameFromFace(face));
        // orient face (flip because of y axis reversal, rotate as necessary)
        let reorient = geometry_core_1.Matrix3d.createRowValues(1, 0, 0, 0, -1, 0, 0, 0, 1);
        // Position face correctly (applies to rotation, as well as translation)
        let reposition = geometry_core_1.Matrix3d.createIdentity();
        switch (this.props.face) {
            case Face.Bottom:
                reposition = geometry_core_1.Matrix3d.createRowValues(-1, 0, 0, 0, 1, 0, 0, 0, -1);
                reorient = geometry_core_1.Matrix3d.createRowValues(-1, 0, 0, 0, 1, 0, 0, 0, 1);
                break;
            case Face.Right:
                reposition = geometry_core_1.Matrix3d.createRowValues(0, 0, 1, 0, 1, 0, -1, 0, 0);
                reorient = geometry_core_1.Matrix3d.createRowValues(0, 1, 0, 1, 0, 0, 0, 0, 1);
                break;
            case Face.Left:
                reposition = geometry_core_1.Matrix3d.createRowValues(0, 0, -1, 0, 1, 0, 1, 0, 0);
                reorient = geometry_core_1.Matrix3d.createRowValues(0, -1, 0, -1, 0, 0, 0, 0, 1);
                break;
            case Face.Back:
                reposition = geometry_core_1.Matrix3d.createRowValues(1, 0, 0, 0, 0, 1, 0, -1, 0);
                reorient = geometry_core_1.Matrix3d.createRowValues(-1, 0, 0, 0, 1, 0, 0, 0, 1);
                break;
            case Face.Front:
                reposition = geometry_core_1.Matrix3d.createRowValues(1, 0, 0, 0, 0, -1, 0, 1, 0);
                reorient = geometry_core_1.Matrix3d.createRowValues(1, 0, 0, 0, -1, 0, 0, 0, 1);
                break;
        }
        const repositioned = rotMatrix.multiplyMatrixMatrix(reposition);
        const vect = repositioned.multiplyXYZ(0, 0, this._faceWidth);
        const m = repositioned.multiplyMatrixMatrix(reorient);
        const list = [
            m.at(0, 0), -m.at(1, 0), m.at(2, 0), 0,
            m.at(0, 1), -m.at(1, 1), m.at(2, 1), 0,
            m.at(0, 2), -m.at(1, 2), m.at(2, 2), 0,
            vect.at(0), -vect.at(1), vect.at(2) - this._faceWidth /* move back faceWidth so face is on screen level */, 1,
        ];
        const transform = `matrix3d(${list.join(", ")})`;
        const s = Object.assign({ transform, 
            // eslint-disable-next-line @typescript-eslint/naming-convention
            WebkitTransform: transform }, style);
        return (React.createElement("div", Object.assign({ style: s, "data-testid": `core-cube-face-${face}`, className: classes, ref: (e) => { this._faceWidth = (e && e.clientWidth / 2) || 0; } }, props), children));
    }
    getCSSClassNameFromFace(face) {
        let className = "";
        // istanbul ignore else
        if (face !== Face.None)
            className = `cube-${face}`;
        return className;
    }
}
exports.CubeFace = CubeFace;
//# sourceMappingURL=Cube.js.map