"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Dialog
 */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Dialog = exports.DialogAlignment = exports.DialogButtonStyle = exports.DialogButtonType = void 0;
require("./Dialog.scss");
const classnames = require("classnames");
const React = require("react");
const ui_abstract_1 = require("@bentley/ui-abstract");
const DivWithOutsideClick_1 = require("../base/DivWithOutsideClick");
const UiCore_1 = require("../UiCore");
const FocusTrap_1 = require("../focustrap/FocusTrap");
// cspell:ignore focustrap
/** Enum for button types. Determines button label, and default button style.
 * @public
 */
var DialogButtonType;
(function (DialogButtonType) {
    DialogButtonType["None"] = "";
    DialogButtonType["Close"] = "close";
    DialogButtonType["OK"] = "ok";
    DialogButtonType["Cancel"] = "cancel";
    DialogButtonType["Yes"] = "yes";
    DialogButtonType["No"] = "no";
    DialogButtonType["Retry"] = "retry";
    DialogButtonType["Next"] = "next";
    DialogButtonType["Previous"] = "previous";
})(DialogButtonType = exports.DialogButtonType || (exports.DialogButtonType = {}));
/** Enum for button style.
 * @public
 */
var DialogButtonStyle;
(function (DialogButtonStyle) {
    DialogButtonStyle["None"] = "";
    DialogButtonStyle["Primary"] = "uicore-buttons-primary";
    DialogButtonStyle["Hollow"] = "uicore-buttons-hollow";
    DialogButtonStyle["Blue"] = "uicore-buttons-blue";
})(DialogButtonStyle = exports.DialogButtonStyle || (exports.DialogButtonStyle = {}));
/** Enum for dialog alignment
 * @public
 */
var DialogAlignment;
(function (DialogAlignment) {
    DialogAlignment["TopLeft"] = "top-left";
    DialogAlignment["Top"] = "top";
    DialogAlignment["TopRight"] = "top-right";
    DialogAlignment["Left"] = "left";
    DialogAlignment["Center"] = "center";
    DialogAlignment["Right"] = "right";
    DialogAlignment["BottomLeft"] = "bottom-left";
    DialogAlignment["Bottom"] = "bottom";
    DialogAlignment["BottomRight"] = "bottom-right";
})(DialogAlignment = exports.DialogAlignment || (exports.DialogAlignment = {}));
/**
 * Dialog React component with optional resizing and dragging functionality
 * @public
 */
class Dialog extends React.Component {
    constructor(props) {
        super(props);
        this._parentDocument = document;
        this._containerRef = React.createRef();
        this.handleRefSet = (containerDiv) => {
            if (containerDiv)
                this._parentDocument = containerDiv.ownerDocument;
        };
        this._handleKeyUp = (event) => {
            if (event.key === ui_abstract_1.SpecialKey.Escape && this.props.opened && this.props.onEscape) {
                this.props.onEscape();
            }
        };
        this._handleContainerPointerDown = (event) => {
            if (!this.props.modal) {
                if (this.props.onModelessPointerDown && this.props.modelessId)
                    this.props.onModelessPointerDown(event, this.props.modelessId);
            }
        };
        this._handleStartResizeRight = (event) => {
            event.preventDefault();
            this.setState({ rightResizing: true });
            const parentWindow = this.getParentWindow();
            parentWindow.addEventListener("pointermove", this._handlePointerMove, true);
        };
        this._handleStartResizeDown = (event) => {
            event.preventDefault();
            this.setState({ downResizing: true });
            const parentWindow = this.getParentWindow();
            parentWindow.addEventListener("pointermove", this._handlePointerMove, true);
        };
        this._handleStartResizeDownRight = (event) => {
            event.preventDefault();
            this.setState({ downResizing: true, rightResizing: true });
            const parentWindow = this.getParentWindow();
            parentWindow.addEventListener("pointermove", this._handlePointerMove, true);
        };
        this._handleStartMove = (event) => {
            if (!this.props.movable)
                return;
            event.preventDefault();
            // istanbul ignore else
            if (this._containerRef.current) {
                const rect = this._containerRef.current.getBoundingClientRect();
                const grabOffsetX = event.clientX - rect.left;
                const grabOffsetY = event.clientY - rect.top;
                this.setState({
                    grabOffsetX,
                    grabOffsetY,
                    moving: true,
                });
                const parentWindow = this.getParentWindow();
                parentWindow.addEventListener("pointermove", this._handlePointerMove, true);
            }
        };
        this._handlePointerMove = (event) => {
            if ((!this.props.resizable && !this.props.movable) || !this._containerRef.current)
                return;
            const { minWidth, maxWidth, minHeight, maxHeight, movable } = this.props;
            let { x, y, width, height } = this.state;
            // istanbul ignore else
            if (x === undefined) { // if x is undefined, so is y, width, and height
                const rect = this._containerRef.current.getBoundingClientRect();
                width = rect.width, height = rect.height, x = rect.left, y = rect.top;
            }
            if (this.props.resizable) {
                if (this.state.rightResizing) {
                    const pointerX = event.clientX;
                    width = pointerX - x;
                    // istanbul ignore else
                    if (typeof minWidth === "number")
                        width = Math.max(width, minWidth);
                    if (maxWidth !== undefined && typeof maxWidth === "number")
                        width = Math.min(width, maxWidth);
                }
                if (this.state.downResizing) {
                    const pointerY = event.clientY;
                    height = pointerY - y;
                    // istanbul ignore else
                    if (typeof minHeight === "number")
                        height = Math.max(height, minHeight);
                    if (maxHeight !== undefined && typeof maxHeight === "number")
                        height = Math.min(height, maxHeight);
                }
                this.setState({ width, height });
            }
            if (movable && this.state.moving) {
                x = event.clientX - this.state.grabOffsetX;
                y = event.clientY - this.state.grabOffsetY;
                this.setState({ x, y, positionSet: true });
            }
        };
        this._handlePointerUp = (_event) => {
            if (!this.props.movable && !this.props.resizable)
                return;
            // istanbul ignore else
            if (this._containerRef.current) {
                this.setState({
                    rightResizing: false,
                    downResizing: false,
                    moving: false,
                    grabOffsetX: 0,
                    grabOffsetY: 0,
                });
            }
            const parentWindow = this.getParentWindow();
            parentWindow.removeEventListener("pointermove", this._handlePointerMove, true);
        };
        this.state = {
            rightResizing: false,
            downResizing: false,
            moving: false,
            grabOffsetX: 0,
            grabOffsetY: 0,
            positionSet: props.x !== undefined || props.y !== undefined,
        };
    }
    getParentWindow() {
        var _a;
        // istanbul ignore next
        return (_a = this._parentDocument.defaultView) !== null && _a !== void 0 ? _a : window;
    }
    componentWillUnmount() {
        const parentWindow = this.getParentWindow();
        parentWindow.removeEventListener("pointerup", this._handlePointerUp, true);
        parentWindow.removeEventListener("pointermove", this._handlePointerMove, true);
        this._parentDocument.removeEventListener("keyup", this._handleKeyUp, true);
    }
    componentDidMount() {
        const parentWindow = this.getParentWindow();
        parentWindow.addEventListener("pointerup", this._handlePointerUp, true);
        this._parentDocument.addEventListener("keyup", this._handleKeyUp, true);
    }
    render() {
        const _a = this.props, { opened, title, footer, buttonCluster, onClose, onEscape, onOutsideClick, // eslint-disable-line @typescript-eslint/no-unused-vars
        minWidth, minHeight, x, y, width, height, maxHeight, maxWidth, backgroundStyle, titleStyle, footerStyle, style, contentStyle, contentClassName, modal, resizable, movable, className, alignment, inset, trapFocus, modelessId, onModelessPointerDown, // eslint-disable-line @typescript-eslint/no-unused-vars
        hideHeader, header } = _a, props = __rest(_a, ["opened", "title", "footer", "buttonCluster", "onClose", "onEscape", "onOutsideClick", "minWidth", "minHeight", "x", "y", "width", "height", "maxHeight", "maxWidth", "backgroundStyle", "titleStyle", "footerStyle", "style", "contentStyle", "contentClassName", "modal", "resizable", "movable", "className", "alignment", "inset", "trapFocus", "modelessId", "onModelessPointerDown", "hideHeader", "header"]);
        const containerStyle = {
            margin: "",
            left: x, top: y,
            width, height,
        };
        if (this.state.x !== undefined || this.state.y !== undefined) {
            // istanbul ignore else
            if (this.state.x !== undefined) {
                containerStyle.marginLeft = "0";
                containerStyle.marginRight = "0";
                containerStyle.left = this.state.x;
            }
            // istanbul ignore else
            if (this.state.y !== undefined) {
                containerStyle.marginTop = "0";
                containerStyle.marginBottom = "0";
                containerStyle.top = this.state.y;
            }
        }
        if (this.state.width !== undefined || this.state.height !== undefined) {
            // istanbul ignore else
            if (this.state.width !== undefined)
                containerStyle.width = this.state.width;
            // istanbul ignore else
            if (this.state.height !== undefined)
                containerStyle.height = this.state.height;
        }
        const minMaxStyle = {};
        minMaxStyle.minWidth = (typeof minWidth === "number") ? `${minWidth}px` : minWidth;
        minMaxStyle.minHeight = (typeof minHeight === "number") ? `${minHeight}px` : minHeight;
        if (maxWidth !== undefined)
            minMaxStyle.maxWidth = (typeof maxWidth === "number") ? `${maxWidth}px` : maxWidth;
        if (maxHeight !== undefined)
            minMaxStyle.maxHeight = (typeof maxHeight === "number") ? `${maxHeight}px` : maxHeight;
        const buttons = this.getFooterButtons(this.props);
        const footerElement = footer || (buttons.length > 0 && React.createElement("div", { className: "core-dialog-buttons" }, buttons));
        const divStyle = Object.assign(Object.assign({}, backgroundStyle), style);
        const headerElement = header || (React.createElement("div", { className: classnames("core-dialog-head", { "core-dialog-movable": movable }), "data-testid": "core-dialog-head", onPointerDown: this._handleStartMove },
            React.createElement("div", { className: "core-dialog-title", "data-testid": "core-dialog-title", style: titleStyle }, title),
            React.createElement("button", { className: "core-focus-trap-ignore-initial core-dialog-close icon icon-close", "data-testid": "core-dialog-close", onClick: onClose })));
        return (React.createElement("div", Object.assign({ ref: this.handleRefSet, className: classnames("core-dialog", !modal && "core-dialog-hidden", opened && "core-dialog-opened", className), style: divStyle, "data-testid": "core-dialog-root" }, props), opened &&
            React.createElement(DivWithOutsideClick_1.DivWithOutsideClick, { onOutsideClick: onOutsideClick },
                React.createElement(FocusTrap_1.FocusTrap, { active: trapFocus && modal, returnFocusOnDeactivate: true },
                    React.createElement("div", { className: classnames("core-dialog-container", this.getCSSClassNameFromAlignment(alignment)), style: Object.assign(Object.assign({}, containerStyle), minMaxStyle), "data-testid": "core-dialog-container", onPointerDown: this._handleContainerPointerDown },
                        React.createElement("div", { className: "core-dialog-area", ref: this._containerRef, style: minMaxStyle },
                            !hideHeader &&
                                headerElement,
                            React.createElement("div", { className: classnames("core-dialog-content", { "core-dialog-content-no-inset": !inset }, contentClassName), style: contentStyle }, this.props.children),
                            footerElement &&
                                React.createElement("div", { className: "core-dialog-footer", style: footerStyle }, footerElement),
                            React.createElement("div", { className: classnames("core-dialog-drag", "core-dialog-drag-right", { "core-dialog-drag-enabled": resizable }), "data-testid": "core-dialog-drag-right", onPointerDown: this._handleStartResizeRight }),
                            React.createElement("div", { className: classnames("core-dialog-drag", "core-dialog-drag-bottom-mid", { "core-dialog-drag-enabled": resizable }), "data-testid": "core-dialog-drag-bottom", onPointerDown: this._handleStartResizeDown }),
                            React.createElement("div", { className: classnames("core-dialog-drag", "core-dialog-drag-bottom-right", { "core-dialog-drag-enabled": resizable }), "data-testid": "core-dialog-drag-bottom-right", onPointerDown: this._handleStartResizeDownRight })))))));
    }
    getCSSClassNameFromAlignment(alignment) {
        let className = "";
        // Drop the alignment CSS class if the Dialog has been sized or moved.
        if (this.state.positionSet)
            return "";
        // istanbul ignore next
        if (!alignment)
            alignment = DialogAlignment.Center;
        switch (alignment) {
            case DialogAlignment.TopLeft:
                className = "core-dialog-top-left";
                break;
            case DialogAlignment.Top:
                className = "core-dialog-top";
                break;
            case DialogAlignment.TopRight:
                className = "core-dialog-top-right";
                break;
            case DialogAlignment.Left:
                className = "core-dialog-left";
                break;
            case DialogAlignment.Center:
                className = "core-dialog-center";
                break;
            case DialogAlignment.Right:
                className = "core-dialog-right";
                break;
            case DialogAlignment.BottomLeft:
                className = "core-dialog-bottom-left";
                break;
            case DialogAlignment.Bottom:
                className = "core-dialog-bottom";
                break;
            case DialogAlignment.BottomRight:
                className = "core-dialog-bottom-right";
                break;
        }
        return className;
    }
    getFooterButtons(props) {
        const buttons = [];
        if (props.buttonCluster) {
            props.buttonCluster.forEach((button, index) => {
                let buttonText = "";
                let buttonClass = classnames("core-dialog-button", `dialog-button-${button.type}`, button.className);
                switch (button.type) {
                    case DialogButtonType.OK:
                        buttonText = UiCore_1.UiCore.translate("dialog.ok");
                        buttonClass = classnames(buttonClass, button.buttonStyle || "uicore-buttons-primary");
                        break;
                    case DialogButtonType.Retry:
                        buttonText = UiCore_1.UiCore.translate("dialog.retry");
                        buttonClass = classnames(buttonClass, button.buttonStyle || "uicore-buttons-primary");
                        break;
                    case DialogButtonType.Yes:
                        buttonText = UiCore_1.UiCore.translate("dialog.yes");
                        buttonClass = classnames(buttonClass, button.buttonStyle || "uicore-buttons-primary");
                        break;
                    case DialogButtonType.No:
                        buttonText = UiCore_1.UiCore.translate("dialog.no");
                        buttonClass = classnames(buttonClass, button.buttonStyle || "uicore-buttons-hollow");
                        break;
                    case DialogButtonType.Cancel:
                        buttonText = UiCore_1.UiCore.translate("dialog.cancel");
                        buttonClass = classnames(buttonClass, button.buttonStyle || "uicore-buttons-hollow");
                        break;
                    case DialogButtonType.Close:
                        buttonText = UiCore_1.UiCore.translate("dialog.close");
                        buttonClass = classnames(buttonClass, button.buttonStyle || "uicore-buttons-hollow");
                        break;
                    case DialogButtonType.Next:
                        buttonText = UiCore_1.UiCore.translate("dialog.next");
                        buttonClass = classnames(buttonClass, button.buttonStyle || "uicore-buttons-primary");
                        break;
                    case DialogButtonType.Previous:
                        buttonText = UiCore_1.UiCore.translate("dialog.previous");
                        buttonClass = classnames(buttonClass, button.buttonStyle || "uicore-buttons-primary");
                        break;
                }
                if (button.label)
                    buttonText = button.label;
                buttons.push(React.createElement("button", { className: buttonClass, disabled: button.disabled, key: index.toString(), onClick: button.onClick }, buttonText));
            });
        }
        return buttons;
    }
}
exports.Dialog = Dialog;
Dialog.defaultProps = {
    alignment: DialogAlignment.Center,
    minWidth: 300,
    minHeight: 100,
    width: "50%",
    hideHeader: false,
    resizable: false,
    movable: false,
    modal: true,
    inset: true,
    trapFocus: false,
};
//# sourceMappingURL=Dialog.js.map