"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module ElementSeparator
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElementSeparator = void 0;
require("./ElementSeparator.scss");
const React = require("react");
const react_1 = require("react"); // eslint-disable-line no-duplicate-imports
const classnames = require("classnames");
const Orientation_1 = require("../enums/Orientation");
const useThrottledFn_1 = require("../utils/hooks/useThrottledFn");
const UiCore_1 = require("../UiCore");
function getCurrentGlobalPosition(orientation, e) {
    return orientation === Orientation_1.Orientation.Horizontal ? e.clientX : e.clientY;
}
const useConditionalCleanup = (condition, cleanup) => {
    const conditionRef = react_1.useRef(condition);
    const cleanupRef = react_1.useRef(cleanup);
    conditionRef.current = condition;
    cleanupRef.current = cleanup;
    react_1.useEffect(() => {
        return () => {
            if (conditionRef.current)
                cleanupRef.current();
        };
    }, []);
};
const useElementSeparatorPointerHandler = ({ onResizeHandleDragChanged, onResizeHandleHoverChanged, isResizeHandleBeingDragged, isResizeHandleHovered, movableArea, ratio, orientation, onRatioChanged, }) => {
    const globalPosition = react_1.useRef(0);
    const pointerOutOfBounds = react_1.useRef(false);
    const [isElementDragged, setIsDragged] = react_1.useState(false);
    const [isElementHovered, setIsHovered] = react_1.useState(false);
    const isGroupDragged = isResizeHandleBeingDragged !== null && isResizeHandleBeingDragged !== void 0 ? isResizeHandleBeingDragged : isElementDragged;
    const isGroupHovered = isResizeHandleHovered !== null && isResizeHandleHovered !== void 0 ? isResizeHandleHovered : isElementHovered;
    useConditionalCleanup(isElementDragged && !!onResizeHandleDragChanged, () => onResizeHandleDragChanged(false));
    useConditionalCleanup(isElementHovered && !!onResizeHandleHoverChanged, () => onResizeHandleHoverChanged(false));
    if (isGroupHovered && pointerOutOfBounds.current)
        pointerOutOfBounds.current = false;
    const stopDrag = react_1.useCallback(() => {
        // istanbul ignore else
        if (isGroupDragged) {
            setIsDragged(false);
            if (onResizeHandleDragChanged)
                onResizeHandleDragChanged(false);
        }
    }, [isGroupDragged, onResizeHandleDragChanged]);
    const startDrag = react_1.useCallback((e) => {
        globalPosition.current = getCurrentGlobalPosition(orientation, e);
        // istanbul ignore else
        if (!isGroupDragged) {
            setIsDragged(true);
            if (onResizeHandleDragChanged)
                onResizeHandleDragChanged(true);
        }
    }, [isGroupDragged, orientation, onResizeHandleDragChanged]);
    const onPointerUp = react_1.useCallback(() => {
        stopDrag();
    }, [stopDrag]);
    const onThrottledPointerMove = useThrottledFn_1.useThrottledFn((e) => {
        if (!movableArea) {
            stopDrag();
            return;
        }
        const currentPosition = getCurrentGlobalPosition(orientation, e);
        const positionChange = currentPosition - globalPosition.current;
        // Should not need to recalculate if position on our movement axis does not change
        if (Math.abs(positionChange) < 1)
            return;
        const currentLocalPosition = movableArea * ratio + positionChange;
        const newRatio = currentLocalPosition / movableArea;
        globalPosition.current = currentPosition;
        if (pointerOutOfBounds.current || !onRatioChanged)
            return;
        const result = onRatioChanged(newRatio);
        if (result && result.ratio === ratio && !isGroupHovered && !pointerOutOfBounds.current)
            pointerOutOfBounds.current = true;
    }, 16, [stopDrag, isGroupHovered, ratio, movableArea, onRatioChanged, orientation]);
    react_1.useEffect(() => {
        return () => onThrottledPointerMove.cancel();
    }, [onThrottledPointerMove]);
    react_1.useLayoutEffect(() => {
        if (isElementDragged) {
            document.addEventListener("pointerup", onPointerUp);
            document.addEventListener("pointermove", onThrottledPointerMove);
        }
        return () => {
            document.removeEventListener("pointerup", onPointerUp);
            document.removeEventListener("pointermove", onThrottledPointerMove);
        };
    }, [isElementDragged, onPointerUp, onThrottledPointerMove]);
    const onPointerDown = react_1.useCallback((e) => {
        if (!isGroupDragged) {
            startDrag(e);
        }
        else {
            stopDrag();
        }
    }, [isGroupDragged, startDrag, stopDrag]);
    const onPointerOver = react_1.useCallback(() => {
        // istanbul ignore next
        if (isGroupHovered)
            return;
        setIsHovered(true);
        if (onResizeHandleHoverChanged)
            onResizeHandleHoverChanged(true);
    }, [isGroupHovered, onResizeHandleHoverChanged]);
    const onPointerOut = react_1.useCallback(() => {
        // istanbul ignore next
        if (!isGroupHovered)
            return;
        setIsHovered(false);
        if (onResizeHandleHoverChanged)
            onResizeHandleHoverChanged(false);
    }, [isGroupHovered, onResizeHandleHoverChanged]);
    return {
        isHovered: isGroupHovered,
        isDragged: isGroupDragged,
        onPointerDown,
        onPointerOver,
        onPointerOut,
    };
};
function getStyle(orientation, separatorSize) {
    separatorSize = separatorSize || 30;
    if (orientation === Orientation_1.Orientation.Horizontal)
        return {
            width: separatorSize,
            margin: `0px ${-Math.floor(separatorSize / 2)}px`,
        };
    return {
        height: separatorSize,
        margin: `${-Math.floor(separatorSize / 2)}px 1px`,
    };
}
/** A movable button, which allows to change the ratio between left element and right element
 * @public
 */
// eslint-disable-next-line @typescript-eslint/naming-convention
const ElementSeparator = (props) => {
    const label = react_1.useRef(UiCore_1.UiCore.translate("elementSeparator.label"));
    const [hasHoverHappened, setHasHoverHappened] = react_1.useState(false);
    const { isDragged, isHovered, onPointerDown, onPointerOver, onPointerOut } = useElementSeparatorPointerHandler(props);
    const isHoverNeeded = isHovered || isDragged;
    if (!hasHoverHappened && isHoverNeeded)
        setHasHoverHappened(isHoverNeeded);
    // This is done to avoid fade-out animation when first rendering.
    const unhoverClass = hasHoverHappened ? "core-element-separator-group-unhovered" : "";
    const classNames = classnames("core-element-separator", (props.orientation === Orientation_1.Orientation.Horizontal) ? "core-element-separator--horizontal" : "core-element-separator--vertical", props.className, isHoverNeeded ? "core-element-separator-group-hovered" : unhoverClass);
    const orientation = props.orientation;
    const separatorSize = props.separatorSize;
    const style = react_1.useMemo(() => getStyle(orientation, separatorSize), [orientation, separatorSize]);
    const styles = Object.assign(Object.assign({}, style), props.style);
    return (React.createElement("button", { style: styles, className: classNames, onPointerDown: onPointerDown, onPointerOver: onPointerOver, onPointerOut: onPointerOut, "aria-label": label.current }));
};
exports.ElementSeparator = ElementSeparator;
//# sourceMappingURL=ElementSeparator.js.map