"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Focus
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.isNavigationKey = exports.ItemKeyboardNavigator = void 0;
const ui_abstract_1 = require("@bentley/ui-abstract");
const Orientation_1 = require("../enums/Orientation");
/** Keyboard Navigator for parent components
 * @internal
 */
class ItemKeyboardNavigator {
    constructor(onFocusItem, onActivateItem) {
        this.onFocusItem = onFocusItem;
        this.onActivateItem = onActivateItem;
        this._itemCount = 0;
        this._orientation = Orientation_1.Orientation.Horizontal;
        this._allowWrap = true;
        this._direction = new Map([
            [ui_abstract_1.SpecialKey.ArrowLeft, -1],
            [ui_abstract_1.SpecialKey.ArrowUp, -1],
            [ui_abstract_1.SpecialKey.ArrowRight, 1],
            [ui_abstract_1.SpecialKey.ArrowDown, 1],
        ]);
    }
    /** The item count */
    get itemCount() { return this._itemCount; }
    set itemCount(count) { this._itemCount = count; }
    /** The primary orientation */
    get orientation() { return this._orientation; }
    set orientation(orientation) { this._orientation = orientation; }
    /** The allow wrap property controls whether the movement will stop at the first and last items or wrap  */
    get allowWrap() { return this._allowWrap; }
    set allowWrap(v) { this._allowWrap = v; }
    /** Called when the arrow keys that run perpendicular to the primary orientation are pressed */
    get crossAxisArrowKeyHandler() { return this._crossAxisArrowKeyHandler; }
    set crossAxisArrowKeyHandler(v) { this._crossAxisArrowKeyHandler = v; }
    /** Handle KeyDown on items */
    handleKeyDownEvent(event, index) {
        const key = event.key;
        switch (key) {
            case ui_abstract_1.SpecialKey.Home:
                event.preventDefault();
                // Activate first item
                this.focusFirstItem();
                break;
            case ui_abstract_1.SpecialKey.End:
                event.preventDefault();
                // Activate last item
                this.focusLastItem();
                break;
            // Up and down are in keydown
            // because we need to prevent page scroll >:)
            case ui_abstract_1.SpecialKey.ArrowUp:
            case ui_abstract_1.SpecialKey.ArrowDown:
                this.determineOrientation(event, index);
                break;
            case ui_abstract_1.SpecialKey.Enter:
            case ui_abstract_1.SpecialKey.Space:
                this.activateItem(index);
                break;
        }
    }
    /** Handle KeyUp on items */
    handleKeyUpEvent(event, index) {
        const key = event.key;
        switch (key) {
            case ui_abstract_1.SpecialKey.ArrowLeft:
            case ui_abstract_1.SpecialKey.ArrowRight:
                this.determineOrientation(event, index);
                break;
        }
    }
    focusFirstItem() {
        this.onFocusItem(0);
    }
    focusLastItem() {
        const index = this._itemCount - 1;
        this.onFocusItem(index);
    }
    /** When an item list's orientation is set to vertical,
     * only up and down arrow should function.
     * In all other cases only left and right arrow function.
     */
    determineOrientation(event, index) {
        const key = event.key;
        const vertical = this._orientation === Orientation_1.Orientation.Vertical;
        let proceed = false;
        if (vertical) {
            if (key === ui_abstract_1.SpecialKey.ArrowUp || key === ui_abstract_1.SpecialKey.ArrowDown) {
                event.preventDefault();
                proceed = true;
            }
            else if (this.crossAxisArrowKeyHandler && (key === ui_abstract_1.SpecialKey.ArrowLeft || key === ui_abstract_1.SpecialKey.ArrowRight)) {
                this.crossAxisArrowKeyHandler(key === ui_abstract_1.SpecialKey.ArrowRight);
            }
        }
        else {
            if (key === ui_abstract_1.SpecialKey.ArrowLeft || key === ui_abstract_1.SpecialKey.ArrowRight) {
                proceed = true;
            }
            else if (this.crossAxisArrowKeyHandler && (key === ui_abstract_1.SpecialKey.ArrowUp || key === ui_abstract_1.SpecialKey.ArrowDown)) {
                this.crossAxisArrowKeyHandler(key === ui_abstract_1.SpecialKey.ArrowDown);
            }
        }
        if (proceed) {
            this.switchItemOnArrowPress(event, index);
        }
    }
    /** Either focus the next, previous, first, or last item depending on key pressed
     */
    switchItemOnArrowPress(event, index) {
        // Add or subtract depending on key pressed
        const pressed = event.key;
        const targetDirection = this._direction.get(pressed);
        // istanbul ignore else
        if (targetDirection) {
            const newIndex = index + targetDirection;
            if (0 <= newIndex && newIndex < this._itemCount) {
                this.onFocusItem(newIndex);
            }
            else {
                // istanbul ignore else
                if (this._allowWrap) {
                    if (pressed === ui_abstract_1.SpecialKey.ArrowLeft || pressed === ui_abstract_1.SpecialKey.ArrowUp) {
                        this.focusLastItem();
                    }
                    else {
                        this.focusFirstItem();
                    }
                }
            }
        }
    }
    activateItem(index) {
        this.onActivateItem(index);
    }
}
exports.ItemKeyboardNavigator = ItemKeyboardNavigator;
/** Determines if a KeyboardEvent.key is an Item Navigation key
 * @internal
 */
function isNavigationKey(key) {
    return (ui_abstract_1.isArrowKey(key) || key === ui_abstract_1.SpecialKey.Home || key === ui_abstract_1.SpecialKey.End || key === ui_abstract_1.SpecialKey.Space || key === ui_abstract_1.SpecialKey.Enter);
}
exports.isNavigationKey = isNavigationKey;
//# sourceMappingURL=ItemKeyboardNavigator.js.map