"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Form
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Form = exports.FormContext = void 0;
require("./Form.scss");
const React = require("react");
const Button_1 = require("../button/Button");
const UiCore_1 = require("../UiCore");
const Field_1 = require("./Field");
/** React context used by Form as a Provider and by the Fields as Consumers and updaters.
 * @beta
 */
// eslint-disable-next-line @typescript-eslint/naming-convention
exports.FormContext = React.createContext(undefined);
exports.FormContext.displayName = "ui-core:FormContext";
/** Component used to create a Form using supplied properties to specify the fields of the Form.
 * @example
 * // Example of using a form as contents of a modal dialog.
 * export class ExampleForm extends React.Component {
 *   public static open() {
 *   const form = new ExampleForm({});
 *   ModalDialogManager.openDialog(form.render());
 * }
 *
 * protected async handleSubmit(values: FieldValues): Promise<void> {
 *   await this.processFormSubmission(values);
 *   ModalDialogManager.closeDialog();
 *   const msg = JSON.stringify(values);
 *   IModelApp.notifications.outputMessage(new NotifyMessageDetails(OutputMessagePriority.Info, "Form Submitted", msg));
 * }
 *
 * protected async processFormSubmission(values: FieldValues): Promise<void> {
 *   // if error occurs during async processing throw an Error and return error message back to form.
 *   throw new Error("Error processing form");
 * }
 *
 * protected handleCancel() {
 *   ModalDialogManager.closeDialog();
 * }
 *
 * public render() {
 *   const fields: FieldDefinitions = {
 *     Name: {
 *       label: this._nameLabel,
 *       editor: "textbox",
 *       value: "John Smith",
 *     },
 *     PickList: {
 *       label: this._pickListLabel,
 *       editor: "dropdown",
 *       value: "one",
 *       options: ["one", "two", "three", "four"],
 *     },
 *   };
 *
 *   return (
 *     <div>
 *       <Dialog title="Example Form"
 *         opened={true}
 *         onClose={() => this.handleCancel()}>
 *         <Form handleFormSubmit={(values: FieldValues) => this.handleSubmit(values)}
 *           fields={fields}
 *         />
 *       </Dialog>
 *     </div >
 *   );
 * }
 * @beta
 */
class Form extends React.Component {
    constructor(props) {
        super(props);
        this._submitButtonLabel = UiCore_1.UiCore.translate("form.submitButtonLabel");
        this._errorPrefix = UiCore_1.UiCore.translate("form.errorPrefix");
        this._errorSuffix = UiCore_1.UiCore.translate("form.errorSuffix");
        this._setValues = (values) => {
            this.setState((prevState) => ({ values: Object.assign(Object.assign({}, prevState.values), values) }));
        };
        this._handleSubmit = (event, values) => {
            event.preventDefault();
            // istanbul ignore next
            if (!this.props.handleFormSubmit || !values || 0 === values.length)
                throw new Error("A handleFormSubmit function and values must be defined");
            this.props.handleFormSubmit(values)
                .then(() => {
                // Currently we don't need to do anything if successful. Typically the hosting dialog will close when the
                // submission is successful.
                // console.log(rtnMsg);
            })
                .catch((reason) => {
                this.setState({ submitSuccess: false, errorMsg: reason.message });
            });
        };
        // set initial values for field values
        const values = {};
        Object.keys(props.fields).forEach((key) => values[key] = props.fields[key].value);
        this.state = {
            errorMsg: "",
            values,
        };
    }
    /**
     * Returns whether there is an errorMsg message in the state/context
     * @param errorMsg - The form errorMsg
     */
    haveError(errorMsg) {
        return errorMsg && errorMsg.length;
    }
    render() {
        const { submitSuccess, errorMsg } = this.state;
        const { fields, submitButtonLabel } = this.props;
        const context = Object.assign(Object.assign({}, this.state), { setValues: this._setValues });
        return (React.createElement(exports.FormContext.Provider, { value: context },
            React.createElement("div", { className: "core-form-wrapper" },
                React.createElement("form", { onSubmit: (event) => this._handleSubmit(event, this.state.values), noValidate: true },
                    React.createElement("div", { className: "core-form-container" },
                        Object.keys(fields).map((key) => React.createElement(Field_1.Field, Object.assign({ key: key, id: key }, this.props.fields[key]))),
                        React.createElement("div", { className: "core-form-footer" },
                            React.createElement("div", { className: "core-form-buttons" },
                                React.createElement(Button_1.Button, { type: "submit", buttonType: Button_1.ButtonType.Blue }, submitButtonLabel ? submitButtonLabel : this._submitButtonLabel)),
                            submitSuccess === false && this.haveError(errorMsg) && (React.createElement("div", { className: "core-form-alert", role: "alert" }, `${this._errorPrefix}-${errorMsg} ${this._errorSuffix}`))))))));
    }
}
exports.Form = Form;
//# sourceMappingURL=Form.js.map