"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Inputs
 */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NumberInput = void 0;
require("./NumberInput.scss");
const classnames = require("classnames");
const React = require("react");
const Input_1 = require("../Input");
const WebFontIcon_1 = require("../../icons/WebFontIcon");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ForwardRefNumberInput = React.forwardRef(function ForwardRefNumberInput(props, ref) {
    const { containerClassName, value, min, max, precision, format, parse, onChange, onBlur, onKeyDown, step, snap, showTouchButtons } = props, otherProps = __rest(props, ["containerClassName", "value", "min", "max", "precision", "format", "parse", "onChange", "onBlur", "onKeyDown", "step", "snap", "showTouchButtons"]);
    const currentValueRef = React.useRef(value);
    /**
   * Used internally to parse the argument x to it's numeric representation.
   * If the argument cannot be converted to finite number returns 0; If a
   * "precision" prop is specified uses it round the number with that
   * precision (no fixed precision here because the return value is float, not
   * string).
   */
    const parseInternal = React.useCallback((x) => {
        let n;
        if (parse)
            n = parse(x);
        if (undefined === n || null === n) {
            n = parseFloat(x);
            if (isNaN(n) || !isFinite(n)) {
                n = 0;
            }
        }
        const localPrecision = undefined === precision ? 10 : precision;
        const q = Math.pow(10, localPrecision);
        const localMin = undefined === min ? Number.MIN_SAFE_INTEGER : min;
        const localMax = undefined === max ? Number.MAX_SAFE_INTEGER : max;
        n = Math.min(Math.max(n, localMin), localMax);
        n = Math.round(n * q) / q;
        return n;
    }, [parse, precision, min, max]);
    /**
   * This is used internally to format a number to its display representation.
   * It will invoke the format function if one is provided.
   */
    const formatInternal = React.useCallback((num) => {
        const localPrecision = undefined === precision || null === precision ? 0 : precision;
        const str = undefined === num || null === num ? "" : num.toFixed(localPrecision);
        if (format)
            return format(num, str);
        return str;
    }, [format, precision]);
    const [formattedValue, setFormattedValue] = React.useState(() => formatInternal(value));
    // See if new initialValue props have changed since component mounted
    React.useEffect(() => {
        currentValueRef.current = value;
        const currentFormattedValue = formatInternal(currentValueRef.current);
        setFormattedValue(currentFormattedValue);
    }, [formatInternal, value]);
    const handleChange = React.useCallback((event) => {
        setFormattedValue(event.currentTarget.value);
    }, []);
    const updateValue = React.useCallback((newVal) => {
        const newFormattedVal = formatInternal(newVal);
        onChange && onChange(newVal, newFormattedVal);
        setFormattedValue(newFormattedVal);
    }, [onChange, formatInternal]);
    const updateValueFromString = React.useCallback((strValue) => {
        const newVal = parseInternal(strValue);
        updateValue(newVal);
    }, [parseInternal, updateValue]);
    const handleBlur = React.useCallback((event) => {
        const newVal = parseInternal(event.target.value);
        onBlur && onBlur(event);
        updateValue(newVal);
    }, [parseInternal, updateValue, onBlur]);
    const getIncrementValue = React.useCallback((increment) => {
        if (typeof step === "function") {
            const stepVal = step(increment ? "up" : "down");
            return stepVal ? stepVal : 1;
        }
        return !step ? 1 : step;
    }, [step]);
    /**
   * The internal method that actually sets the new value on the input
   */
    const applyStep = React.useCallback((increment) => {
        const incrementValue = getIncrementValue(increment);
        let num = parseInternal(formattedValue) + (increment ? incrementValue : -incrementValue);
        if (snap) {
            num = Math.round(num / incrementValue) * incrementValue;
        }
        const localMin = undefined === min ? Number.MIN_SAFE_INTEGER : min;
        const localMax = undefined === max ? Number.MAX_SAFE_INTEGER : max;
        num = Math.min(Math.max(num, localMin), localMax);
        updateValue(num);
    }, [formattedValue, getIncrementValue, max, min, parseInternal, snap, updateValue]);
    const handleKeyDown = React.useCallback((event) => {
        // istanbul ignore else
        if (event.key === ui_abstract_1.SpecialKey.Enter) {
            updateValueFromString(event.currentTarget.value);
            event.preventDefault();
        }
        else if (event.key === ui_abstract_1.SpecialKey.Escape) {
            setFormattedValue(formatInternal(currentValueRef.current));
            event.preventDefault();
        }
        else if (event.key === ui_abstract_1.SpecialKey.ArrowDown) {
            applyStep(false);
            event.preventDefault();
        }
        else if (event.key === ui_abstract_1.SpecialKey.ArrowUp) {
            applyStep(true);
            event.preventDefault();
        }
        onKeyDown && onKeyDown(event);
    }, [applyStep, formatInternal, updateValueFromString, onKeyDown]);
    const handleDownClick = React.useCallback((event) => {
        applyStep(false);
        event.preventDefault();
    }, [applyStep]);
    const handleUpClick = React.useCallback((event) => {
        applyStep(true);
        event.preventDefault();
    }, [applyStep]);
    const containerClasses = classnames("core-number-input-container", containerClassName, showTouchButtons && "core-number-buttons-for-touch");
    return (React.createElement("div", { className: containerClasses },
        React.createElement(Input_1.Input, Object.assign({ ref: ref, value: formattedValue, onChange: handleChange, onKeyDown: handleKeyDown, onBlur: handleBlur }, otherProps)),
        React.createElement("div", { className: classnames("core-number-input-buttons-container", showTouchButtons && "core-number-buttons-for-touch") },
            React.createElement("div", { className: "core-number-input-button core-number-input-button-up", tabIndex: -1, onClick: handleUpClick },
                React.createElement(WebFontIcon_1.WebFontIcon, { iconName: "icon-caret-up" })),
            React.createElement("div", { className: "core-number-input-button core-number-input-button-down", tabIndex: -1, onClick: handleDownClick },
                React.createElement(WebFontIcon_1.WebFontIcon, { iconName: "icon-caret-down" })))));
});
/** Input component for numbers with up and down buttons to increment and decrement the value.
 * @beta
 */
exports.NumberInput = ForwardRefNumberInput;
//# sourceMappingURL=NumberInput.js.map