"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListboxItem = exports.Listbox = exports.ListboxContext = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
const React = require("react");
const classnames = require("classnames");
require("./Listbox.scss");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_abstract_1 = require("@bentley/ui-abstract");
/**
 * Context set up by listbox for use by `ListboxItems` .
 * @alpha
 */
// istanbul ignore next
exports.ListboxContext = React.createContext({ onListboxValueChange: (_newValue) => { } }); // eslint-disable-line @typescript-eslint/naming-convention
function makeId(...args) {
    return args.filter((val) => val != null).join("--");
}
function getOptionValueArray(childNodes) {
    return React.Children.toArray(childNodes).filter((node) => React.isValidElement(node) && node.props.value).map((optionNode) => optionNode.props);
}
function processKeyboardNavigation(optionValues, itemIndex, key) {
    let keyProcessed = false;
    let newIndex = itemIndex >= 0 ? itemIndex : 0;
    // Note: In aria example Page Up/Down just moves up or down by one item. See https://www.w3.org/TR/wai-aria-practices-1.1/examples/listbox/js/listbox.js
    if (key === ui_abstract_1.SpecialKey.ArrowDown || key === ui_abstract_1.SpecialKey.PageDown) {
        for (let i = itemIndex + 1; i < optionValues.length; i++) {
            if (!optionValues[i].disabled) {
                newIndex = i;
                break;
            }
        }
        keyProcessed = true;
    }
    else if (key === ui_abstract_1.SpecialKey.ArrowUp || key === ui_abstract_1.SpecialKey.PageUp) {
        for (let i = itemIndex - 1; i >= 0; i--) {
            if (!optionValues[i].disabled) {
                newIndex = i;
                break;
            }
        }
        keyProcessed = true;
    }
    else if (key === ui_abstract_1.SpecialKey.Home) {
        for (let i = 0; i < optionValues.length; i++) {
            if (!optionValues[i].disabled) {
                newIndex = i;
                break;
            }
        }
        keyProcessed = true;
    }
    else if (key === ui_abstract_1.SpecialKey.End) {
        for (let i = optionValues.length - 1; i >= 0; i--) {
            if (!optionValues[i].disabled) {
                newIndex = i;
                break;
            }
        }
        keyProcessed = true;
    }
    return [newIndex, keyProcessed];
}
/**
 * Single select `Listbox` component
 * @alpha
 */
function Listbox(props) {
    const { ariaLabel, ariaLabelledBy, id, children, selectedValue, className, onListboxValueChange, onKeyPress } = props, otherProps = __rest(props, ["ariaLabel", "ariaLabelledBy", "id", "children", "selectedValue", "className", "onListboxValueChange", "onKeyPress"]);
    const listRef = React.useRef(null);
    const [listId] = React.useState(() => { return id !== null && id !== void 0 ? id : bentleyjs_core_1.Guid.createValue(); });
    const optionValues = React.useMemo(() => getOptionValueArray(children), [children]);
    const classes = React.useMemo(() => classnames("core-listbox", className), [className]);
    const [currentValue, setCurrentValue] = React.useState(selectedValue);
    const [focusValue, setFocusValue] = React.useState(currentValue);
    React.useEffect(() => {
        setCurrentValue(selectedValue);
        setFocusValue(selectedValue);
    }, [selectedValue]);
    const scrollTopRef = React.useRef(0);
    const handleValueChange = React.useCallback((newValue, isControlOrCommandPressed) => {
        // istanbul ignore else
        if (newValue !== currentValue) {
            setCurrentValue(newValue);
            setFocusValue(newValue);
            if (onListboxValueChange)
                onListboxValueChange(newValue, isControlOrCommandPressed);
        }
    }, [setCurrentValue, currentValue, onListboxValueChange]);
    const focusOption = React.useCallback((itemIndex) => {
        // istanbul ignore else
        if (itemIndex >= 0 && itemIndex < optionValues.length) {
            const newSelection = optionValues[itemIndex];
            const listElement = listRef.current;
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
            const optionToFocus = listElement.querySelector(`li[data-value="${newSelection.value}"]`);
            // istanbul ignore else
            if (optionToFocus && listElement) {
                let newScrollTop = listElement.scrollTop;
                // istanbul ignore next
                if (listElement.scrollHeight > listElement.clientHeight) {
                    const scrollBottom = listElement.clientHeight + listElement.scrollTop;
                    const elementBottom = optionToFocus.offsetTop + optionToFocus.offsetHeight;
                    if (elementBottom > scrollBottom) {
                        newScrollTop = elementBottom - listElement.clientHeight;
                    }
                    else if (optionToFocus.offsetTop < listElement.scrollTop) {
                        newScrollTop = optionToFocus.offsetTop;
                    }
                    scrollTopRef.current = newScrollTop;
                }
                setFocusValue(newSelection.value);
            }
        }
    }, [optionValues]);
    const handleKeyDown = React.useCallback((event) => {
        if (optionValues.length < 1)
            return;
        const itemIndex = (undefined === focusValue) ? -1 : optionValues.findIndex((optionValue) => ((optionValue.value === focusValue)));
        if (event.key === ui_abstract_1.SpecialKey.Space) {
            event.preventDefault();
            // istanbul ignore else
            if (focusValue)
                handleValueChange(focusValue, event.getModifierState("Control") || event.getModifierState("Meta")); // Control or Command
            return;
        }
        else {
            const [newItemIndex, keyProcessed] = processKeyboardNavigation(optionValues, itemIndex, event.key);
            // istanbul ignore else
            if (keyProcessed) {
                event.preventDefault();
                focusOption(newItemIndex);
                return;
            }
        }
        // istanbul ignore else
        if (onKeyPress)
            onKeyPress(event);
    }, [focusValue, optionValues, focusOption, onKeyPress, handleValueChange]);
    const isInitialMount = React.useRef(true);
    React.useEffect(() => {
        const list = listRef.current;
        if (isInitialMount.current) {
            isInitialMount.current = false;
            if (undefined !== focusValue) {
                const itemIndex = optionValues.findIndex((optionValue) => (optionValue.value === focusValue));
                focusOption(itemIndex);
            }
        }
        else {
            list.scrollTop = scrollTopRef.current;
        }
    }, [focusValue, focusOption, optionValues]);
    // istanbul ignore next
    const handleOnScroll = React.useCallback((_event) => {
        if (listRef.current)
            scrollTopRef.current = listRef.current.scrollTop;
    }, []);
    // istanbul ignore next
    const handleOnFocus = React.useCallback((_event) => {
        if (!focusValue || 0 === focusValue.length) {
            if (currentValue) {
                setFocusValue(currentValue);
            }
            else {
                if (optionValues.length > 0)
                    setFocusValue(optionValues[0].value);
            }
        }
    }, [currentValue, focusValue, optionValues]);
    return React.createElement("ul", Object.assign({ className: classes, "aria-labelledby": ariaLabel ? undefined : ariaLabelledBy, "aria-label": ariaLabel, 
        // An element that contains or owns all the listbox options has role
        // listbox.
        // https://www.w3.org/TR/wai-aria-practices-1.2/#Listbox
        role: "listbox", 
        // https://www.w3.org/TR/wai-aria-practices-1.2/examples/listbox/listbox-collapsible.html
        tabIndex: 0, "aria-activedescendant": makeId(currentValue, listId) }, otherProps, { ref: listRef, id: listId, onKeyDown: handleKeyDown, onScroll: handleOnScroll, "data-value": currentValue, "data-focusvalue": focusValue, onFocus: handleOnFocus }),
        React.createElement(exports.ListboxContext.Provider, { value: {
                listboxValue: currentValue,
                focusValue,
                listboxId: listId,
                onListboxValueChange: handleValueChange,
                listboxRef: listRef,
            } }, children));
}
exports.Listbox = Listbox;
/**
 * `ListboxItem` component.
 * @alpha
 */
function ListboxItem(props) {
    const { children, value, className, disabled } = props, otherProps = __rest(props, ["children", "value", "className", "disabled"]);
    const { listboxValue, focusValue, listboxId, onListboxValueChange, } = React.useContext(exports.ListboxContext);
    const hasFocus = focusValue === value;
    const classes = React.useMemo(() => classnames("core-listbox-item", hasFocus && "focused", className), [className, hasFocus]);
    const itemRef = React.useRef(null);
    const isSelected = listboxValue === value;
    const handleClick = React.useCallback((event) => {
        var _a, _b;
        event.preventDefault();
        // istanbul ignore next
        const selectedValue = (_b = (_a = event.currentTarget) === null || _a === void 0 ? void 0 : _a.dataset) === null || _b === void 0 ? void 0 : _b.value;
        // istanbul ignore else
        if (undefined !== selectedValue) {
            onListboxValueChange(selectedValue, event.ctrlKey);
        }
    }, [onListboxValueChange]);
    const getItemId = React.useCallback(() => {
        return makeId(value, listboxId);
    }, [listboxId, value]);
    return (
    // eslint-disable-next-line jsx-a11y/click-events-have-key-events
    React.createElement("li", Object.assign({ "aria-selected": isSelected, "aria-disabled": disabled || undefined, 
        // Each option in the listbox has role `option` and is a DOM descendant
        // of the element with role `listbox`.
        // https://www.w3.org/TR/wai-aria-practices-1.2/#Listbox
        role: "option", className: classes }, otherProps, { ref: itemRef, id: getItemId(), "data-value": value, onClick: handleClick }), children));
}
exports.ListboxItem = ListboxItem;
//# sourceMappingURL=Listbox.js.map