"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module SearchBox
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SearchBox = void 0;
require("./SearchBox.scss");
const classnames = require("classnames");
const React = require("react");
const UiCore_1 = require("../UiCore");
const ui_abstract_1 = require("@bentley/ui-abstract");
/**
 * Input box for entering text to search for.
 * The SearchBox has an icon right-justified and bounded by the box and shows a Search or Clear icon.
 * @public
 */
class SearchBox extends React.Component {
    constructor(props) {
        super(props);
        this._inputElement = null;
        this._timeoutId = 0;
        /** @internal */
        this.state = { value: this.props.initialValue || "" };
        /** Wrapper for onValueChanged to make sure we don't call search unless the new value is different from the previous value */
        this._onValueChanged = (value, previousValue) => {
            // istanbul ignore else
            if (value === previousValue)
                return;
            this.setState((_prevState) => {
                return {
                    value,
                };
            }, () => { this.props.onValueChanged(this.state.value); });
        };
        this._trackChange = (_event) => {
            let value = "";
            const previousValue = this.state.value;
            // istanbul ignore else
            if (this._inputElement)
                value = this._inputElement.value;
            if (this.props.valueChangedDelay) {
                this._unsetTimeout();
                this._timeoutId = window.setTimeout(() => { this._onValueChanged(value, previousValue); }, this.props.valueChangedDelay);
            }
            else {
                this._onValueChanged(value, previousValue);
            }
        };
        this._handleKeyDown = (e) => {
            switch (e.key) {
                case ui_abstract_1.SpecialKey.Escape:
                    // istanbul ignore else
                    if (this.props.onEscPressed)
                        this.props.onEscPressed();
                    break;
                case ui_abstract_1.SpecialKey.Enter:
                    // istanbul ignore else
                    if (this.props.onEnterPressed)
                        this.props.onEnterPressed();
                    break;
            }
        };
        this._handleIconClick = (_event) => {
            // istanbul ignore else
            if (this._inputElement) {
                const clear = this.state.value !== "";
                this._inputElement.value = "";
                // istanbul ignore else
                if (clear && this.props.onClear)
                    this.props.onClear();
                this._inputElement.focus();
            }
            this._trackChange();
        };
        this._unsetTimeout = () => {
            if (this._timeoutId) {
                window.clearTimeout(this._timeoutId);
                this._timeoutId = 0;
            }
        };
    }
    /** @internal */
    render() {
        const searchClassName = classnames("core-searchbox", this.props.className);
        const emptyString = this.state.value === "";
        const iconClassName = classnames("core-searchbox-icon", "icon", {
            "icon-search": emptyString,
            "icon-close": !emptyString,
        });
        const buttonTitle = UiCore_1.UiCore.translate(emptyString ? "general.search" : "general.clear");
        return (React.createElement("div", { className: searchClassName, style: this.props.style },
            React.createElement("input", { defaultValue: this.props.initialValue, ref: (el) => { this._inputElement = el; }, onChange: this._trackChange, onKeyDown: this._handleKeyDown, onPaste: this._trackChange, onCut: this._trackChange, placeholder: this.props.placeholder ? this.props.placeholder : UiCore_1.UiCore.translate("general.search"), role: "searchbox", "data-testid": "core-searchbox-input" }),
            React.createElement("div", { className: "core-searchbox-button", onClick: this._handleIconClick, role: "button", tabIndex: -1, title: buttonTitle },
                React.createElement("span", { className: iconClassName }))));
    }
    componentWillUnmount() {
        this._unsetTimeout();
    }
    focus() {
        // istanbul ignore else
        if (this._inputElement)
            this._inputElement.focus();
    }
}
exports.SearchBox = SearchBox;
//# sourceMappingURL=SearchBox.js.map