"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Settings
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SettingsContainer = exports.useSaveBeforeActivatingNewSettingsTab = exports.useSaveBeforeClosingSettingsContainer = void 0;
require("./SettingsContainer.scss");
const React = require("react");
const VerticalTabs_1 = require("../tabs/VerticalTabs");
const ui_abstract_1 = require("@bentley/ui-abstract");
/*  ---------------------------------------------------------------------------------------------------
// A typical implementation of a saveFunction callback
const saveChanges = React.useCallback((afterSaveFunction: (args: any) => void, args?: any) => {
  if (dataIsDirty) {
    // prompt user to save changes passing in function and arguments to call after saving changes.
    ModalDialogManager.openDialog(<CustomSavePromptModalDialog customProps={customProps}
      onDialogCloseArgs={args} onDialogClose={afterSaveFunction} />);
    return;
  }
  afterSaveFunction(args);
}, []);
---------------------------------------------------------------------------------------------------------- */
/** Hook to use within Settings Page component to allow saving the current page's data before the Setting Container is closed.
 * @beta
 */
function useSaveBeforeClosingSettingsContainer(settingsManager, saveFunction) {
    React.useEffect(() => {
        const handleProcessSettingsContainerClose = ({ closeFunc, closeFuncArgs }) => {
            saveFunction(closeFunc, closeFuncArgs);
        };
        return settingsManager.onProcessSettingsContainerClose.addListener(handleProcessSettingsContainerClose);
    }, [saveFunction, settingsManager]);
}
exports.useSaveBeforeClosingSettingsContainer = useSaveBeforeClosingSettingsContainer;
/** Hook to use within Settings Page component to allow saving the current page's data before loading to the requested Setting Tab's page.
 * @beta
 */
function useSaveBeforeActivatingNewSettingsTab(settingsManager, saveFunction) {
    React.useEffect(() => {
        const handleProcessSettingsTabActivation = ({ tabSelectionFunc, requestedSettingsTabId }) => {
            saveFunction(tabSelectionFunc, requestedSettingsTabId);
        };
        return settingsManager.onProcessSettingsTabActivation.addListener(handleProcessSettingsTabActivation);
    }, [saveFunction, settingsManager]);
}
exports.useSaveBeforeActivatingNewSettingsTab = useSaveBeforeActivatingNewSettingsTab;
/**
 * SettingsContainer component that displays Setting Tabs on Left and the P
 * Note: that SettingsContainer is not rendered if tabs array is empty
 * @beta
 */
const SettingsContainer = ({ tabs, onSettingsTabSelected, currentSettingsTab, settingsManager, showHeader }) => {
    // sort the tabs based on itemPriority
    tabs = tabs.sort((a, b) => a.itemPriority - b.itemPriority);
    const [openTab, setOpenTab] = React.useState(() => {
        if (currentSettingsTab && !ui_abstract_1.ConditionalBooleanValue.getValue(currentSettingsTab.isDisabled))
            return currentSettingsTab;
        else
            return tabs[0];
    });
    const processTabSelection = React.useCallback((tab) => {
        if (ui_abstract_1.ConditionalBooleanValue.getValue(tab.isDisabled))
            return;
        onSettingsTabSelected && onSettingsTabSelected(tab);
        setOpenTab(tab);
    }, [onSettingsTabSelected]);
    const processTabSelectionById = React.useCallback((tabId) => {
        const tabToActivate = tabs.find((tab) => tab.tabId === tabId);
        tabToActivate && processTabSelection(tabToActivate);
    }, [processTabSelection, tabs]);
    const onActivateTab = React.useCallback((tabIndex) => {
        const selectedTab = tabs[tabIndex];
        if (openTab && openTab.pageWillHandleCloseRequest)
            settingsManager.onProcessSettingsTabActivation.emit({ requestedSettingsTabId: selectedTab.tabId, tabSelectionFunc: processTabSelectionById });
        else
            processTabSelection(selectedTab);
    }, [openTab, processTabSelection, processTabSelectionById, settingsManager, tabs]);
    React.useEffect(() => {
        const handleActivateSettingsTab = ({ settingsTabId }) => {
            const idToFind = settingsTabId.toLowerCase();
            let tabToActivate = tabs.find((tab) => tab.tabId.toLowerCase() === idToFind);
            if (!tabToActivate)
                tabToActivate = tabs.find((tab) => tab.label.toLowerCase() === idToFind);
            // istanbul ignore else
            if (tabToActivate) {
                if (openTab && openTab.pageWillHandleCloseRequest)
                    settingsManager.onProcessSettingsTabActivation.emit({ requestedSettingsTabId: tabToActivate.tabId, tabSelectionFunc: processTabSelectionById });
                else
                    processTabSelection(tabToActivate);
            }
        };
        return settingsManager.onActivateSettingsTab.addListener(handleActivateSettingsTab);
    }, [openTab, processTabSelection, processTabSelectionById, settingsManager, settingsManager.onActivateSettingsTab, tabs]);
    React.useEffect(() => {
        const handleSettingsContainerClose = ({ closeFunc, closeFuncArgs }) => {
            if (openTab && openTab.pageWillHandleCloseRequest)
                settingsManager.onProcessSettingsContainerClose.emit({ closeFunc, closeFuncArgs });
            else
                closeFunc(closeFuncArgs);
        };
        return settingsManager.onCloseSettingsContainer.addListener(handleSettingsContainerClose);
    }, [openTab, processTabSelection, settingsManager, settingsManager.onActivateSettingsTab, tabs]);
    const labels = tabs.map((tab) => {
        return { label: tab.label, subLabel: tab.subLabel, icon: tab.icon,
            tooltip: tab.tooltip, tabId: tab.tabId, disabled: ui_abstract_1.ConditionalBooleanValue.getValue(tab.isDisabled) };
    });
    const activeIndex = tabs.findIndex((tab) => tab.tabId === openTab.tabId);
    return (React.createElement("div", { className: "core-settings-container" },
        React.createElement("div", { className: "core-settings-container-left" },
            React.createElement(VerticalTabs_1.VerticalTabs, { labels: labels, activeIndex: activeIndex, onActivateTab: onActivateTab })),
        React.createElement("div", { className: "core-settings-container-right" },
            showHeader && React.createElement("div", { className: "core-settings-container-right-header" },
                React.createElement("span", { className: "core-settings-container-main-header" }, openTab.label),
                openTab.subLabel && React.createElement("span", { className: "core-settings-container-main-sub-header" }, openTab.subLabel)),
            React.createElement("div", { className: "core-settings-container-right-contents" }, openTab.page))));
};
exports.SettingsContainer = SettingsContainer;
//# sourceMappingURL=SettingsContainer.js.map