"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Slider
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Slider = void 0;
require("./Slider.scss");
const classnames = require("classnames");
const React = require("react");
const react_compound_slider_1 = require("react-compound-slider");
const BodyText_1 = require("../text/BodyText");
const useRefState_1 = require("../utils/hooks/useRefState");
const Tooltip_1 = require("../tooltip/Tooltip");
/**
 * Slider React component displays a range slider.
 * The Slider component uses various components from the
 * [react-compound-slider](https://www.npmjs.com/package/react-compound-slider)
 * package internally.
 * @public
 */
function Slider(props) {
    const { className, style, min, max, values, step, mode, formatMin, formatMax, onChange, onUpdate, onSlideStart, onSlideEnd, showTicks, showTickLabels, formatTick, getTickCount, getTickValues, includeTicksInWidth, reversed, disabled, showMinMax, minImage, maxImage, showTooltip, tooltipBelow, formatTooltip, } = props;
    const domain = [min, max];
    const multipleValues = values.length > 1;
    const containerClassNames = classnames("core-slider-container", className, disabled && "core-disabled", showTickLabels && "core-slider-tickLabels", includeTicksInWidth && "core-slider-includeTicksInWidth");
    const sliderClassNames = classnames("core-slider", showMinMax && "core-slider-minMax");
    const internalFormatTooltip = React.useCallback((value) => {
        if (formatTooltip)
            return formatTooltip(value);
        const actualStep = Math.abs(step !== null && step !== void 0 ? step : 1);
        if (Number.isInteger(actualStep))
            return value.toFixed(0);
        const stepString = actualStep.toString();
        const decimalIndex = stepString.indexOf(".");
        const numDecimals = actualStep.toString().length - (decimalIndex + 1);
        return value.toFixed(numDecimals);
    }, [formatTooltip, step]);
    return (React.createElement("div", { className: containerClassNames, style: style },
        showMinMax &&
            React.createElement(MinMax, { value: min, testId: "core-slider-min", image: minImage, format: formatMin }),
        React.createElement(react_compound_slider_1.Slider, { domain: domain, step: step, mode: mode, values: values, reversed: reversed, disabled: disabled, onChange: onChange, onUpdate: onUpdate, onSlideStart: onSlideStart, onSlideEnd: onSlideEnd, className: sliderClassNames, "data-testid": "core-slider" },
            React.createElement(react_compound_slider_1.Rail, null, ({ getRailProps }) => React.createElement(Rails, { getRailProps: getRailProps })),
            React.createElement(react_compound_slider_1.Tracks, { right: false, left: !multipleValues }, ({ tracks, activeHandleID, getEventData, getTrackProps }) => (React.createElement("div", { className: "slider-tracks" }, tracks.map(({ id, source, target }) => (React.createElement(TooltipTrack, { key: id, source: source, target: target, activeHandleID: activeHandleID, getEventData: getEventData, getTrackProps: getTrackProps, showTooltip: showTooltip !== null && showTooltip !== void 0 ? showTooltip : true, tooltipBelow: tooltipBelow, formatTooltip: internalFormatTooltip, multipleValues: multipleValues })))))),
            showTicks &&
                React.createElement(react_compound_slider_1.Ticks, { values: getTickValues && getTickValues(), count: getTickCount && getTickCount() }, ({ ticks }) => (React.createElement("div", { className: "slider-ticks", "data-testid": "core-slider-ticks" }, ticks.map((tick, index) => (React.createElement(Tick, { key: tick.id, tick: tick, count: ticks.length, index: index, formatTick: formatTick, showTickLabels: showTickLabels })))))),
            React.createElement(react_compound_slider_1.Handles, null, ({ handles, activeHandleID, getHandleProps }) => (React.createElement("div", { className: "slider-handles" }, handles.map((handle) => (React.createElement(Handle, { key: handle.id, domain: domain, handle: handle, isActive: handle.id === activeHandleID, getHandleProps: getHandleProps, showTooltip: showTooltip, tooltipBelow: tooltipBelow, formatTooltip: internalFormatTooltip, disabled: disabled }))))))),
        showMinMax &&
            React.createElement(MinMax, { value: max, testId: "core-slider-max", image: maxImage, format: formatMax })));
}
exports.Slider = Slider;
/** MinMax component for Slider */
function MinMax(props) {
    const { value, testId, image, format } = props;
    let element;
    const displayValue = format !== undefined ? format(value) : value;
    if (image)
        element = React.createElement(React.Fragment, null, image);
    else
        element = React.createElement(BodyText_1.BodyText, { "data-testid": testId }, displayValue);
    return element;
}
/** Rails component for Slider */
function Rails(props) {
    const { getRailProps } = props;
    return (React.createElement("div", Object.assign({ className: "core-slider-rail" }, getRailProps()),
        React.createElement("div", { className: "core-slider-rail-inner" })));
}
/** TooltipTrack component for Slider */
function TooltipTrack(props) {
    const { source, target, activeHandleID, showTooltip, tooltipBelow, multipleValues, formatTooltip, getTrackProps, getEventData, } = props;
    const [percent, setPercent] = React.useState(null);
    const [tooltipTargetRef, tooltipTarget] = useRefState_1.useRefState();
    // istanbul ignore next
    const onPointerMove = (e) => {
        if (activeHandleID) {
            setPercent(null);
        }
        else {
            const state = getEventData(e.nativeEvent);
            setPercent(state.percent);
        }
    };
    // istanbul ignore next
    const onPointerLeave = () => {
        setPercent(null);
    };
    let tooltipText = "";
    if (multipleValues) {
        const sourceValue = formatTooltip(source.value);
        const targetValue = formatTooltip(target.value);
        tooltipText = `${sourceValue} : ${targetValue}`;
    }
    // istanbul ignore next
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { className: "core-slider_track-tooltip-container", style: { left: `${percent}%` }, ref: tooltipTargetRef }),
        React.createElement(Tooltip_1.Tooltip, { target: tooltipTarget, placement: tooltipBelow ? "bottom" : "top", visible: !activeHandleID && percent !== null && showTooltip && multipleValues }, tooltipText),
        React.createElement("div", Object.assign({ className: "core-slider-track", "data-testid": "core-slider-track", style: { left: `${source.percent}%`, width: `${target.percent - source.percent}%` }, onPointerMove: onPointerMove, onPointerLeave: onPointerLeave }, getTrackProps()),
            React.createElement("div", { className: "core-slider-track-inner" }))));
}
/** Tick component for Slider */
function Tick(props) {
    const { tick, count, showTickLabels, formatTick } = props;
    return (React.createElement("div", null,
        React.createElement("div", { className: "core-slider-tick-mark", style: { left: `${tick.percent}%` } }),
        showTickLabels &&
            React.createElement("div", { className: "core-slider-tick-label", style: { marginLeft: `${-(100 / count) / 2}%`, width: `${100 / count}%`, left: `${tick.percent}%` } }, formatTick !== undefined ? formatTick(tick.value) : tick.value)));
}
/** Handle component for Slider */
function Handle(props) {
    const { domain: [min, max], handle: { id, value, percent }, isActive, disabled, getHandleProps, showTooltip, tooltipBelow, formatTooltip, } = props;
    const [mouseOver, setMouseOver] = React.useState(false);
    const [focused, setFocused] = React.useState(false);
    const [tooltipTargetRef, tooltipTarget] = useRefState_1.useRefState();
    // istanbul ignore next
    const onMouseEnter = () => {
        setMouseOver(true);
    };
    // istanbul ignore next
    const onMouseLeave = () => {
        setMouseOver(false);
    };
    // istanbul ignore next
    const onFocus = () => {
        setFocused(true);
    };
    // istanbul ignore next
    const onBlur = () => {
        setFocused(false);
    };
    const classNames = classnames("core-slider-handle", disabled && "core-disabled");
    const tooltip = formatTooltip ? formatTooltip(value) : /* istanbul ignore next */ value.toString();
    // istanbul ignore next
    return (React.createElement(React.Fragment, null,
        React.createElement(Tooltip_1.Tooltip, { placement: tooltipBelow ? "bottom" : "top", target: tooltipTarget, visible: (mouseOver || isActive || focused) && !disabled && showTooltip }, tooltip),
        React.createElement("div", Object.assign({ "aria-valuemin": min, "aria-valuemax": max, "aria-valuenow": value, "aria-disabled": disabled, "aria-label": tooltip, className: classNames, "data-testid": "core-slider-handle", ref: tooltipTargetRef, role: "slider", tabIndex: disabled ? -1 : 0, style: { left: `${percent}%` } }, getHandleProps(id, {
            onMouseEnter,
            onMouseLeave,
            onFocus,
            onBlur,
        })))));
}
//# sourceMappingURL=Slider.js.map