"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module SplitButton
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SplitButton = exports.SplitButtonActionType = void 0;
require("./SplitButton.scss");
const classnames = require("classnames");
const React = require("react");
const PopupContextMenu_1 = require("../contextmenu/PopupContextMenu");
const IconComponent_1 = require("../icons/IconComponent");
const ui_abstract_1 = require("@bentley/ui-abstract");
const Button_1 = require("../button/Button");
// TODO: implement
/** @internal */
var SplitButtonActionType;
(function (SplitButtonActionType) {
    SplitButtonActionType[SplitButtonActionType["ContextMenu"] = 0] = "ContextMenu";
    SplitButtonActionType[SplitButtonActionType["List"] = 1] = "List";
})(SplitButtonActionType = exports.SplitButtonActionType || (exports.SplitButtonActionType = {}));
/**
 * SplitButton with an action button on the left and an arrow button that displays a context menu on the right.
 * @public
 */
class SplitButton extends React.Component {
    constructor(props) {
        super(props);
        this._arrowElement = React.createRef();
        this._buttonRef = React.createRef();
        this._closing = false;
        /** @internal */
        this.state = { expanded: false };
        this._handleKeyUp = (event) => {
            if (event.key === ui_abstract_1.SpecialKey.Enter) {
                this.props.onExecute && this.props.onExecute();
            }
            else if (event.key === ui_abstract_1.SpecialKey.ArrowDown && !this.state.expanded) {
                this._closing = false;
                this._open();
            }
            else {
                this._closing = false;
            }
        };
        this._open = () => {
            // istanbul ignore else
            if (!this.state.expanded && !this._closing) {
                this.setState({ expanded: true });
            }
            else {
                this._closing = false;
            }
        };
        this._handleArrowClick = (event) => {
            if (this.state.expanded) {
                event.stopPropagation();
                this.setState({ expanded: false });
                this._closing = false;
            }
            else {
                this._open();
            }
        };
        this._handleClose = (event) => {
            // istanbul ignore else
            if (this._arrowElement.current && this._buttonRef.current) {
                // istanbul ignore next
                if (this.state.expanded && "target" in event && this._arrowElement.current.contains(event.target))
                    this._closing = true;
                this.setState((_prevState) => ({ expanded: false }));
                this._buttonRef.current.focus();
            }
        };
        this.state = {
            expanded: props.initialExpanded !== undefined ? props.initialExpanded : false,
        };
    }
    render() {
        let icon = (React.createElement(React.Fragment, null));
        if (this.props.icon !== undefined) {
            icon = (React.createElement(IconComponent_1.Icon, { iconSpec: this.props.icon }));
        }
        const borderClassName = (this.props.drawBorder && this.props.buttonType === undefined) ? "core-split-button-border" : undefined;
        const typeClassName = this.props.buttonType !== undefined ? Button_1.getButtonTypeClassName(this.props.buttonType) : "core-split-button-default-colors";
        const classNames = classnames("core-split-button", typeClassName, borderClassName, this.state.expanded && "core-expanded", this.props.className);
        const position = this.props.popupPosition !== undefined ? this.props.popupPosition : ui_abstract_1.RelativePosition.BottomLeft;
        return (React.createElement("div", { "data-testid": "core-split-button-root", title: this.props.toolTip, className: classNames, style: this.props.style, tabIndex: 0, onKeyUp: this._handleKeyUp, ref: this._buttonRef, role: "button" },
            React.createElement("div", { "data-testid": "core-split-button-label", onClick: this.props.onClick, className: classnames("core-split-button-label", this.props.drawBorder && "core-split-button-border"), role: "button", tabIndex: -1 },
                icon,
                " ",
                this.props.label),
            React.createElement("div", { className: classnames("core-split-button-divider", this.props.drawBorder && "core-split-button-border") }),
            React.createElement("div", { className: "core-split-button-arrow", ref: this._arrowElement, onClick: this._handleArrowClick, role: "button", tabIndex: -1 },
                React.createElement("div", { className: classnames("core-split-button-arrow-icon", "icon", "icon-chevron-down") }),
                React.createElement(PopupContextMenu_1.PopupContextMenu, { position: position, target: this._buttonRef.current, animate: false, selectedIndex: 0, onSelect: this._handleClose, onOutsideClick: this._handleClose, onEsc: this._handleClose, isOpen: this.state.expanded, autoflip: false }, this.props.children))));
    }
}
exports.SplitButton = SplitButton;
//# sourceMappingURL=SplitButton.js.map