"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Toggle
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Toggle = exports.ToggleButtonType = void 0;
require("./Toggle.scss");
const classnames = require("classnames");
const React = require("react");
const useRefs_1 = require("../utils/hooks/useRefs");
/** Toggle display types
 * @public
 */
var ToggleButtonType;
(function (ToggleButtonType) {
    /** Primary (green) background */
    ToggleButtonType[ToggleButtonType["Primary"] = 0] = "Primary";
    /** Blue background */
    ToggleButtonType[ToggleButtonType["Blue"] = 1] = "Blue";
})(ToggleButtonType = exports.ToggleButtonType || (exports.ToggleButtonType = {}));
const ForwardRefToggle = React.forwardRef(function ForwardRefToggle(props, ref) {
    const inputElement = React.useRef(null);
    const refs = useRefs_1.useRefs(inputElement, ref); // combine ref needed for target with the forwardRef needed by the Parent when parent is a Type Editor.
    const padding = 2;
    const defaultHeight = props.large ? 32 : 21;
    const [height, setHeight] = React.useState(defaultHeight);
    const [width, setWidth] = React.useState(defaultHeight * 2);
    const [checked, setChecked] = React.useState(props.isOn ? true : false);
    const [toggling, setToggling] = React.useState(false);
    React.useEffect(() => {
        if (props.setFocus && inputElement.current)
            inputElement.current.focus();
    }, [props]);
    React.useEffect(() => {
        setChecked(props.isOn ? true : false);
    }, [props.isOn]);
    const handleChange = React.useCallback(() => {
        const newChecked = !checked;
        setToggling(true);
        setChecked(newChecked);
        // istanbul ignore else
        if (props.onChange)
            props.onChange(newChecked);
        setTimeout(() => {
            // istanbul ignore else
            if (inputElement.current)
                setToggling(false);
        }, 250);
    }, [props, checked]);
    const handleBlur = React.useCallback((event) => {
        // istanbul ignore else
        if (props.onBlur)
            props.onBlur(event);
    }, [props]);
    const handleCheckboxBlur = React.useCallback((event) => {
        event.stopPropagation();
    }, []);
    const setHeightFromRef = React.useCallback((el) => {
        if (el !== null) {
            // istanbul ignore next
            if (el.clientHeight > 0 && el.clientWidth > 0) {
                setHeight(el.clientHeight);
                setWidth(el.clientWidth);
            }
        }
    }, []);
    const getOffset = () => {
        return (checked) ? width - height : 0;
    };
    /** Default props */
    const rounded = props.rounded !== undefined ? props.rounded : true;
    const showCheckmark = props.showCheckmark !== undefined ? props.showCheckmark : false;
    const buttonType = props.buttonType !== undefined ? props.buttonType : ToggleButtonType.Blue;
    const halfHeight = height / 2;
    const checkmarkClassName = classnames("core-toggle-checkmark", "icon", "icon-checkmark", showCheckmark && "core-visible", toggling && "core-toggling");
    const toggleStyle = Object.assign({ borderRadius: rounded ? halfHeight : 3, fontSize: halfHeight }, props.style);
    const toggleClassName = classnames("core-toggle", buttonType === ToggleButtonType.Primary && "core-toggle-primary", props.large && "core-toggle-large", rounded && "core-toggle-rounded", props.disabled && "uicore-disabled", props.className);
    const toggleHandleStyle = {
        width: height - (padding * 2),
        transform: `translateX(${getOffset()}px)`,
        top: padding,
        bottom: padding,
        left: padding,
    };
    const handleClassName = classnames("core-toggle-handle", toggling && "core-toggling");
    return (React.createElement("label", { ref: setHeightFromRef, style: toggleStyle, className: toggleClassName, onBlur: handleBlur },
        React.createElement("input", { type: "checkbox", ref: refs, className: "core-toggle-input", checked: checked, disabled: props.disabled, onChange: handleChange, onBlur: handleCheckboxBlur, title: props.title }),
        React.createElement("span", { className: "core-toggle-background" }),
        React.createElement("span", { className: checkmarkClassName }),
        React.createElement("span", { className: handleClassName, style: toggleHandleStyle })));
});
/**
 * Toggle React component to show an "on" or "off" state
 * @public
 */
exports.Toggle = ForwardRefToggle;
//# sourceMappingURL=Toggle.js.map