"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tree
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TreeNode = exports.LEVEL_OFFSET = void 0;
require("./Node.scss");
const classnames = require("classnames");
const React = require("react");
const Checkbox_1 = require("../checkbox/Checkbox");
const CheckBoxState_1 = require("../enums/CheckBoxState");
const Spinner_1 = require("../loading/Spinner");
const ExpansionToggle_1 = require("./ExpansionToggle");
/** Number of pixels the node gets offset per each hierarchy level */
exports.LEVEL_OFFSET = 20;
const EXPANSION_TOGGLE_WIDTH = 24;
/** Presentation React component for a Tree node
 * @public
 */
class TreeNode extends React.Component {
    constructor(props) {
        super(props);
        this._onCheckboxChange = (checked) => {
            if (this.props.checkboxProps && this.props.checkboxProps.onClick && !this.props.checkboxProps.isDisabled)
                this.props.checkboxProps.onClick(checked ? CheckBoxState_1.CheckBoxState.On : CheckBoxState_1.CheckBoxState.Off);
        };
        this._onCheckboxClick = (e) => {
            e.stopPropagation();
        };
        this._onClickExpansionToggle = (e) => {
            e.stopPropagation();
            if (this.props.onClickExpansionToggle)
                this.props.onClickExpansionToggle();
        };
        this._onClick = (e) => {
            e.stopPropagation();
            if (this.props.onClick)
                this.props.onClick(e);
        };
    }
    render() {
        const className = classnames("core-tree-node", this.props.isFocused && "is-focused", this.props.isSelected && "is-selected", this.props.isHoverDisabled && "is-hover-disabled", this.props.className);
        let offset = this.props.level * exports.LEVEL_OFFSET;
        if (!this.props.isLoading && this.props.isLeaf)
            offset += EXPANSION_TOGGLE_WIDTH; // Add expansion toggle/loader width if they're not rendered
        const loader = this.props.isLoading ? (React.createElement("div", { className: "loader" },
            React.createElement(Spinner_1.Spinner, { size: Spinner_1.SpinnerSize.Small }))) : undefined;
        let checkbox;
        if (this.props.checkboxProps) {
            const props = {
                label: "",
                checked: this.props.checkboxProps.state === CheckBoxState_1.CheckBoxState.On,
                indeterminate: this.props.checkboxProps.state === CheckBoxState_1.CheckBoxState.Partial,
                disabled: this.props.checkboxProps.isDisabled,
                title: this.props.checkboxProps.tooltip,
                onClick: this._onCheckboxClick,
                onChange: this._onCheckboxChange,
            };
            if (this.props.renderOverrides && this.props.renderOverrides.renderCheckbox) {
                checkbox = this.props.renderOverrides.renderCheckbox(props);
            }
            else {
                checkbox = (React.createElement(Checkbox_1.Checkbox, Object.assign({}, props, { onChange: (e) => this._onCheckboxChange(e.target.checked), "data-testid": this.createSubComponentTestId("checkbox") })));
            }
        }
        const icon = this.props.icon ? (React.createElement("div", { className: "core-tree-node-icon" }, this.props.icon)) : undefined;
        const toggle = (this.props.isLoading || this.props.isLeaf) ? undefined : (React.createElement(ExpansionToggle_1.ExpansionToggle, { className: "expansion-toggle", "data-testid": this.createSubComponentTestId("expansion-toggle"), onClick: this._onClickExpansionToggle, isExpanded: this.props.isExpanded }));
        return (
        // eslint-disable-next-line jsx-a11y/click-events-have-key-events
        React.createElement("div", { className: className, style: this.props.style, "data-testid": this.props["data-testid"], onClick: this._onClick, onContextMenu: this.props.onContextMenu, onMouseDown: this.props.onMouseDown, onMouseUp: this.props.onMouseUp, onMouseMove: this.props.onMouseMove, role: "treeitem", tabIndex: -1 },
            React.createElement("div", { className: "contents", style: { marginLeft: offset }, "data-testid": this.createSubComponentTestId("contents") },
                loader,
                toggle,
                checkbox,
                icon,
                this.props.label),
            this.props.children));
    }
    createSubComponentTestId(subId) {
        if (!this.props["data-testid"])
            return undefined;
        return `${this.props["data-testid"]}-${subId}`;
    }
}
exports.TreeNode = TreeNode;
//# sourceMappingURL=Node.js.map