"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tree
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tree = void 0;
require("./Tree.scss");
const classnames = require("classnames");
const React = require("react");
const geometry_core_1 = require("@bentley/geometry-core");
/** Presentation React component for a Tree
 * @public
 */
class Tree extends React.PureComponent {
    constructor() {
        super(...arguments);
        this._treeElement = React.createRef();
    }
    get _scrollableContainer() {
        if (!this._treeElement.current)
            return undefined;
        const isScrollable = (element) => {
            const style = window.getComputedStyle(element);
            return style.overflow === "auto" || style.overflow === "scroll"
                || style.overflowY === "auto" || style.overflowY === "scroll"
                || style.overflowX === "auto" || style.overflowX === "scroll";
        };
        let scrollableContainer = this._treeElement.current;
        while (scrollableContainer && !isScrollable(scrollableContainer))
            scrollableContainer = (scrollableContainer.children.length > 0) ? scrollableContainer.children[0] : undefined;
        return scrollableContainer;
    }
    scrollToElement(element) {
        const container = this._scrollableContainer;
        if (!container)
            return;
        // istanbul ignore next
        if (!Element.prototype.scrollTo) { // eslint-disable-line @typescript-eslint/unbound-method
            // workaround for Edge scrollTo issue https://developer.microsoft.com/en-us/microsoft-edge/platform/issues/15534521/
            element.scrollIntoView();
            return;
        }
        const elementBox = element.getBoundingClientRect();
        const elementRange = geometry_core_1.Range2d.createXYXY(elementBox.left, elementBox.top, elementBox.right, elementBox.bottom);
        const containerBox = container.getBoundingClientRect();
        const containerRange = geometry_core_1.Range2d.createXYXY(containerBox.left - container.scrollLeft, containerBox.top - container.scrollTop, containerBox.right - container.scrollLeft, containerBox.bottom - container.scrollTop);
        let left;
        if (container.scrollLeft > 0 && elementRange.high.x <= containerRange.high.x) {
            // always attempt to keep horizontal scroll at 0
            left = 0;
        }
        else if (containerRange.low.x <= elementRange.low.x && containerRange.high.x >= elementRange.high.x) {
            // already visible - no need to scroll to
            left = container.scrollLeft;
        }
        else {
            left = elementRange.low.x - containerRange.low.x;
        }
        let top;
        if (containerRange.low.y <= elementRange.low.y && containerRange.high.y >= elementRange.high.y) {
            // already visible - no need to scroll to
            top = container.scrollTop;
        }
        else {
            top = elementRange.low.y - containerRange.low.y;
        }
        container.scrollTo({ left, top });
    }
    getElementsByClassName(className) {
        if (!this._treeElement.current)
            return [];
        const elems = new Array();
        const collection = this._treeElement.current.getElementsByClassName(className);
        for (let i = 0; i < collection.length; ++i)
            elems.push(collection.item(i));
        return elems;
    }
    setFocusByClassName(selector) {
        let status = false;
        if (this._treeElement.current) {
            const element = this._treeElement.current.querySelector(selector);
            // istanbul ignore else
            if (element && element.focus) {
                element.focus();
                status = true;
            }
        }
        return status;
    }
    render() {
        const className = classnames("core-tree", this.props.className);
        return (React.createElement("div", { ref: this._treeElement, className: className, style: this.props.style, onMouseDown: this.props.onMouseDown, onMouseMove: this.props.onMouseMove, onMouseUp: this.props.onMouseUp, onKeyDown: this.props.onKeyDown, onKeyUp: this.props.onKeyUp, role: "tree", tabIndex: -1 }, this.props.children));
    }
}
exports.Tree = Tree;
//# sourceMappingURL=Tree.js.map