"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module UiSettings
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.UiSetting = void 0;
const UiSettingsStorage_1 = require("./UiSettingsStorage");
/** A Ui Setting with namespace and setting name.
 * @public
 */
class UiSetting {
    /** Constructor
     * @param settingNamespace  Namespace for the setting, passed to UiSettings.
     * @param settingName       Name for the setting, passed to UiSettings.
     * @param getValue          Function for getting the value from the application.
     * @param applyValue        Function for applying the setting value to the application.
     * @param defaultValue      Optional default value if not already stored.
     */
    constructor(settingNamespace, settingName, getValue, applyValue, defaultValue) {
        this.settingNamespace = settingNamespace;
        this.settingName = settingName;
        this.getValue = getValue;
        this.applyValue = applyValue;
        this.defaultValue = defaultValue;
    }
    /** Gets the setting from [[UiSettingsStorage]] */
    async getSetting(storage) {
        return storage.getSetting(this.settingNamespace, this.settingName);
    }
    /** Saves the setting value from the `getValue` function to UiSettings */
    async saveSetting(storage) {
        return storage.saveSetting(this.settingNamespace, this.settingName, this.getValue());
    }
    /** Deletes the setting from UiSettings */
    async deleteSetting(storage) {
        return storage.deleteSetting(this.settingNamespace, this.settingName);
    }
    /** Gets the setting from UiSettings and applies the value using the `applyValue` function */
    async getSettingAndApplyValue(storage) {
        if (this.applyValue) {
            const result = await this.getSetting(storage);
            if (result.status === UiSettingsStorage_1.UiSettingsStatus.Success) {
                this.applyValue(result.setting);
            }
            else if (undefined !== this.defaultValue) {
                this.applyValue(this.defaultValue);
                result.setting = this.defaultValue;
                result.status = UiSettingsStorage_1.UiSettingsStatus.Success;
            }
            return result;
        }
        return { status: UiSettingsStorage_1.UiSettingsStatus.Uninitialized };
    }
}
exports.UiSetting = UiSetting;
//# sourceMappingURL=UiSetting.js.map