"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Utilities
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.IconHelper = void 0;
const React = require("react");
const ui_abstract_1 = require("@bentley/ui-abstract");
const IconComponent_1 = require("../icons/IconComponent");
/** Icon Helper Class used to store the data needed to generate an <Icon> for use in any control that shows an icon.
 * @public
 */
class IconHelper {
    static get reactIconKey() {
        return "#-react-iconspec-node-#";
    }
    /** Returns an <Icon> ReactNode from the many ways an icon can be specified.
     * @param icon abstract icon specification.
     * @param internalData a map that may hold a React.ReactNode stored in an abstract item definition.
     */
    static getIconReactNode(icon, internalData) {
        // istanbul ignore else
        if (!icon)
            return null;
        // istanbul ignore else
        if (React.isValidElement(icon))
            return React.createElement(IconComponent_1.Icon, { iconSpec: icon });
        // istanbul ignore if
        if (!(icon instanceof ui_abstract_1.ConditionalStringValue) && typeof icon !== "string")
            return null;
        const iconString = ui_abstract_1.ConditionalStringValue.getValue(icon);
        if (!iconString)
            return null;
        // istanbul ignore else
        if (iconString === IconHelper.reactIconKey) {
            // istanbul ignore else
            if (internalData)
                return React.createElement(IconComponent_1.Icon, { iconSpec: internalData.get(IconHelper.reactIconKey) });
            return null;
        }
        return React.createElement(IconComponent_1.Icon, { iconSpec: iconString });
    }
    /** Returns an icon definition that can be stored in an abstract item definition. If the iconSpec specifies a ReactNode
     * then the react data is stored in the internalData map and the key to the react data is returned.
     * @param iconSpec any supported variations of how an icon can be defined in an abstract item or IconProps.
     * @param internalData a map supplied by the caller to store away react element if React.ReactNode
     */
    static getIconData(iconSpec, internalData) {
        const icon = (React.isValidElement(iconSpec)) ? IconHelper.reactIconKey : iconSpec;
        if (internalData && typeof icon === "string" && icon === IconHelper.reactIconKey) {
            internalData.set(IconHelper.reactIconKey, iconSpec);
        }
        if (typeof icon === "string" || icon instanceof ui_abstract_1.ConditionalStringValue)
            return icon;
        return "";
    }
}
exports.IconHelper = IconHelper;
//# sourceMappingURL=IconHelper.js.map