"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Utilities
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.useOptionalDisposable = exports.useDisposable = void 0;
const react_1 = require("react");
/**
 * A custom hook which creates a disposable object and manages its disposal on unmount
 * or factory method change.
 * @public
 */
function useDisposable(createDisposable) {
    return useOptionalDisposable(createDisposable);
}
exports.useDisposable = useDisposable;
/**
 * A custom hook which calls the factory method to create a disposable object
 * which might as well be undefined. If the result was a disposable object, the
 * hook takes care of disposing it when necessary.
 * @public
 */
function useOptionalDisposable(createDisposable) {
    const previous = react_1.useRef();
    const value = react_1.useMemo(() => {
        if (previous.current)
            previous.current.dispose();
        previous.current = createDisposable();
        return previous.current;
    }, [createDisposable]);
    react_1.useEffect(() => () => previous.current && previous.current.dispose(), []);
    return value;
}
exports.useOptionalDisposable = useOptionalDisposable;
//# sourceMappingURL=useDisposable.js.map