"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Utilities
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.useOnOutsideClick = void 0;
const React = require("react");
const PointerEvents_1 = require("../../base/PointerEvents");
const Timer_1 = require("../Timer");
/** Invokes onOutsideClick handler when user clicks outside of referenced element.
 * @public
 */
function useOnOutsideClick(onOutsideClick, 
/** Invoked for intermediate events. Return `false` to prevent outside click. */
outsideEventPredicate) {
    const handleMouseEvents = React.useRef(true);
    const handleMouseEventsTimer = React.useRef(new Timer_1.Timer(1000));
    const ref = React.useRef(null);
    const isDownOutside = React.useRef(false);
    React.useEffect(() => {
        const listener = (e) => {
            if (e.type === "touchstart") {
                // Skip mouse event handlers after touch event.
                handleMouseEvents.current = false;
                handleMouseEventsTimer.current.start();
            }
            else if (e.type === "mousedown" && !handleMouseEvents.current) {
                return;
            }
            const isOutsideEvent = !outsideEventPredicate || outsideEventPredicate(e);
            isDownOutside.current = (!!ref.current && (e.target instanceof Node) && !ref.current.contains(e.target)) && isOutsideEvent;
        };
        if (PointerEvents_1.hasPointerEventsSupport()) {
            document.addEventListener("pointerdown", listener);
        }
        else {
            document.addEventListener("mousedown", listener);
            document.addEventListener("touchstart", listener);
        }
        return () => {
            if (PointerEvents_1.hasPointerEventsSupport()) {
                document.removeEventListener("pointerdown", listener);
            }
            else {
                document.removeEventListener("mousedown", listener);
                document.removeEventListener("touchstart", listener);
            }
        };
    }, [outsideEventPredicate]);
    React.useEffect(() => {
        const listener = (e) => {
            if (e.type === "mouseup" && !handleMouseEvents.current) {
                return;
            }
            onOutsideClick && isDownOutside.current && (!outsideEventPredicate || outsideEventPredicate(e)) &&
                ref.current && e.target instanceof Node && !ref.current.contains(e.target) && onOutsideClick();
            isDownOutside.current = false;
        };
        if (PointerEvents_1.hasPointerEventsSupport()) {
            document.addEventListener("pointerup", listener);
        }
        else {
            document.addEventListener("mouseup", listener);
            document.addEventListener("touchend", listener);
        }
        return () => {
            if (PointerEvents_1.hasPointerEventsSupport()) {
                document.removeEventListener("pointerup", listener);
            }
            else {
                document.removeEventListener("mouseup", listener);
                document.removeEventListener("touchend", listener);
            }
        };
    }, [onOutsideClick, outsideEventPredicate]);
    React.useEffect(() => {
        const listener = () => {
            handleMouseEvents.current = true;
        };
        const timer = handleMouseEventsTimer.current;
        timer.setOnExecute(listener);
        return () => {
            timer.setOnExecute(undefined);
        };
    }, []);
    return ref;
}
exports.useOnOutsideClick = useOnOutsideClick;
//# sourceMappingURL=useOnOutsideClick.js.map