"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Utilities
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getToolbarBackdropFilter = exports.getToolbarBoxShadow = exports.getToolbarBackgroundColor = exports.calculateBackdropFilterBlur = exports.calculateBoxShadowOpacity = exports.calculateToolbarOpacity = exports.calculateProximityScale = exports.TOOLBAR_BACKDROP_FILTER_BLUR_DEFAULT = exports.TOOLBAR_BOX_SHADOW_OPACITY_DEFAULT = exports.TOOLBAR_OPACITY_DEFAULT = exports.PROXIMITY_THRESHOLD_DEFAULT = exports.useProximityToMouse = exports.WidgetElementSet = void 0;
const React = require("react");
const Point_1 = require("../Point");
const Rectangle_1 = require("../Rectangle");
/** @internal */
class WidgetElementSet extends Set {
}
exports.WidgetElementSet = WidgetElementSet;
/** Returns the proximity scale associated with the shortest distance from the element(s) to the mouse.
 * @internal
 */
const useProximityToMouse = (elementSet, snap = false, threshold = exports.PROXIMITY_THRESHOLD_DEFAULT) => {
    const [proximityScale, setProximityScale] = React.useState(1.0);
    React.useEffect(() => {
        const handleDocumentPointerMove = (e) => {
            let shortestProximity = Number.MAX_SAFE_INTEGER;
            for (const ref of elementSet) {
                // istanbul ignore else
                if (ref.current) {
                    const clientRect = ref.current.getBoundingClientRect();
                    const rectangle = Rectangle_1.Rectangle.create(clientRect);
                    const point = new Point_1.Point(e.pageX, e.pageY);
                    shortestProximity = Math.min(rectangle.getShortestDistanceToPoint(point), shortestProximity);
                }
            }
            const scale = exports.calculateProximityScale(shortestProximity, snap, threshold);
            if (scale !== proximityScale)
                setProximityScale(scale);
        };
        document.addEventListener("pointermove", handleDocumentPointerMove);
        return () => {
            document.removeEventListener("pointermove", handleDocumentPointerMove);
        };
    }, [elementSet, proximityScale, snap, threshold]);
    return proximityScale;
};
exports.useProximityToMouse = useProximityToMouse;
/** Default proximity threshold.
 * @internal
 */
exports.PROXIMITY_THRESHOLD_DEFAULT = 100;
/** Default toolbar opacity.
 * @internal
 */
exports.TOOLBAR_OPACITY_DEFAULT = 0.5;
/** Default toolbar box-shadow opacity.
 * @internal
 */
exports.TOOLBAR_BOX_SHADOW_OPACITY_DEFAULT = 0.35;
/** Default toolbar backdrop-filter blur.
 * @internal
 */
exports.TOOLBAR_BACKDROP_FILTER_BLUR_DEFAULT = 10;
/** Calculates a proximity scale for further calculations given the proximity and threshold.
 * @internal
 */
const calculateProximityScale = (proximity, snap = false, threshold = exports.PROXIMITY_THRESHOLD_DEFAULT) => {
    let scale = ((proximity < threshold) ? threshold - proximity : 0) / threshold;
    if (snap && scale > 0)
        scale = 1.0;
    return scale;
};
exports.calculateProximityScale = calculateProximityScale;
/** Calculates a toolbar opacity based on a given proximity scale.
 * @internal
 */
const calculateToolbarOpacity = (proximityScale) => {
    return exports.TOOLBAR_OPACITY_DEFAULT * proximityScale;
};
exports.calculateToolbarOpacity = calculateToolbarOpacity;
/** Calculates a box-shadow opacity based on a given proximity scale.
 * @internal
 */
const calculateBoxShadowOpacity = (proximityScale) => {
    return exports.TOOLBAR_BOX_SHADOW_OPACITY_DEFAULT * proximityScale;
};
exports.calculateBoxShadowOpacity = calculateBoxShadowOpacity;
/** Calculates a backdrop-filter blur based on a given proximity scale.
 * @internal
 */
const calculateBackdropFilterBlur = (proximityScale) => {
    return exports.TOOLBAR_BACKDROP_FILTER_BLUR_DEFAULT * proximityScale;
};
exports.calculateBackdropFilterBlur = calculateBackdropFilterBlur;
/** Gets the Toolbar background-color based on a given opacity.
 * @internal
 */
const getToolbarBackgroundColor = (opacity) => {
    return `rgba(var(--buic-background-2-rgb), ${opacity})`;
};
exports.getToolbarBackgroundColor = getToolbarBackgroundColor;
/** Gets the Toolbar box-shadow based on a given opacity.
 * @internal
 */
const getToolbarBoxShadow = (opacity) => {
    return `0 1px 3px 0 rgba(0, 0, 0, ${opacity})`;
};
exports.getToolbarBoxShadow = getToolbarBoxShadow;
/** Gets the Toolbar backdrop-filter based on a given blur amount.
 * @internal
 */
const getToolbarBackdropFilter = (filterBlur) => {
    return `blur(${filterBlur}px)`;
};
exports.getToolbarBackdropFilter = getToolbarBackdropFilter;
//# sourceMappingURL=useProximityToMouse.js.map