"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResizableContainerObserver = exports.ElementResizeObserver = exports.useLayoutResizeObserver = exports.useResizeObserver = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Utilities
 */
const React = require("react");
const ResizeObserverPolyfill_1 = require("./ResizeObserverPolyfill");
const useRefEffect_1 = require("./useRefEffect");
const useRefs_1 = require("./useRefs");
/** Uses ResizeObserver API to notify about element bound changes.
  * @internal
  */
function useResizeObserver(onResize) {
    const [bounds, setBounds] = React.useState({ width: 0, height: 0 });
    const resizeObserver = React.useRef(null);
    const rafRef = React.useRef(0); // set to non-zero when requestAnimationFrame processing is active
    const isMountedRef = React.useRef(false);
    const owningWindowRef = React.useRef(null);
    // istanbul ignore next
    const resizeObserverCleanup = () => {
        var _a;
        if (rafRef.current)
            owningWindowRef.current.cancelAnimationFrame(rafRef.current);
        (_a = resizeObserver.current) === null || _a === void 0 ? void 0 : _a.disconnect();
        resizeObserver.current = null;
    };
    React.useEffect(() => {
        isMountedRef.current = true;
        return () => {
            var _a;
            isMountedRef.current = false;
            if (rafRef.current)
                owningWindowRef.current.cancelAnimationFrame(rafRef.current);
            (_a = owningWindowRef.current) === null || _a === void 0 ? void 0 : _a.removeEventListener("beforeunload", resizeObserverCleanup);
        };
    }, []);
    const processResize = React.useCallback((target) => {
        const newBounds = target.getBoundingClientRect();
        // istanbul ignore else
        if (isMountedRef.current && (bounds.width !== newBounds.width || bounds.height !== newBounds.height)) {
            setBounds(newBounds);
            onResize && onResize(newBounds.width, newBounds.height);
        }
    }, [onResize, bounds.height, bounds.width]);
    const handleResize = React.useCallback((entries) => {
        // istanbul ignore if
        if (!isMountedRef.current || !Array.isArray(entries) || 0 === entries.length) {
            return;
        }
        const target = entries[0].target;
        // istanbul ignore next
        owningWindowRef.current && owningWindowRef.current.removeEventListener("beforeunload", resizeObserverCleanup);
        owningWindowRef.current = target.ownerDocument.defaultView;
        // istanbul ignore else
        if (owningWindowRef.current) {
            owningWindowRef.current.addEventListener("beforeunload", resizeObserverCleanup);
            // using requestAnimationFrame to stop the "ResizeObserver loop completed with undelivered notifications." and
            // "ResizeObserver loop limit exceeded" messages reported to window.onError
            rafRef.current = owningWindowRef.current.requestAnimationFrame(() => processResize(target));
        }
    }, [processResize]);
    const observerRef = useRefEffect_1.useRefEffect((instance) => {
        resizeObserver.current = new ResizeObserverPolyfill_1.ResizeObserver(handleResize);
        if (instance) {
            resizeObserver.current.observe(instance);
            const newBounds = instance.getBoundingClientRect();
            onResize && onResize(newBounds.width, newBounds.height);
            setBounds(newBounds);
        }
        return () => {
            var _a;
            instance && ( /* istanbul ignore next */(_a = resizeObserver.current) === null || _a === void 0 ? void 0 : _a.unobserve(instance));
            resizeObserver.current = null;
        };
    }, [handleResize, onResize]);
    const handleRef = React.useCallback((instance) => {
        const newBounds = instance && instance.getBoundingClientRect();
        // istanbul ignore if
        if (newBounds && (bounds.width !== newBounds.width || bounds.height !== newBounds.height)) {
            setBounds(newBounds);
            onResize && onResize(newBounds.width, newBounds.height);
        }
    }, [bounds.height, bounds.width, onResize]);
    const ref = useRefs_1.useRefs(handleRef, observerRef);
    return ref;
}
exports.useResizeObserver = useResizeObserver;
/** React hook that uses ResizeObserver API to notify about element bound changes. Note: to work similar to ReactResizeDetector
 * width and height are initialized as undefined and only set during after mount if bounds are non-zero. This implementation properly
 * handles observing element in pop-out/child windows.
  * @internal
  */
function useLayoutResizeObserver(inElement, onResize) {
    const inBoundingRect = inElement === null || inElement === void 0 ? void 0 : inElement.getBoundingClientRect();
    const [bounds, setBounds] = React.useState({ width: inBoundingRect === null || inBoundingRect === void 0 ? void 0 : inBoundingRect.width, height: inBoundingRect === null || inBoundingRect === void 0 ? void 0 : inBoundingRect.height });
    const [watchedElement, setWatchedElement] = React.useState(inElement);
    const resizeObserver = React.useRef(null);
    const rafRef = React.useRef(0); // set to non-zero when requestAnimationFrame processing is active
    const isMountedRef = React.useRef(false);
    const owningWindowRef = React.useRef(null);
    // istanbul ignore next
    const resizeObserverCleanup = () => {
        var _a;
        if (rafRef.current)
            owningWindowRef.current.cancelAnimationFrame(rafRef.current);
        (_a = resizeObserver.current) === null || _a === void 0 ? void 0 : _a.disconnect();
        resizeObserver.current = null;
    };
    React.useEffect(() => {
        isMountedRef.current = true;
        setWatchedElement(inElement);
        // istanbul ignore next
        if (inElement) {
            const newBounds = inElement.getBoundingClientRect();
            onResize && onResize(newBounds.width, newBounds.height);
            setBounds(newBounds);
        }
        return () => {
            isMountedRef.current = false;
            (rafRef.current && owningWindowRef.current) && owningWindowRef.current.cancelAnimationFrame(rafRef.current);
            owningWindowRef.current && owningWindowRef.current.removeEventListener("beforeunload", resizeObserverCleanup);
        };
    }, [onResize, inElement]);
    const processResize = React.useCallback((target) => {
        const newBounds = target.getBoundingClientRect();
        // istanbul ignore next
        if (isMountedRef.current && (bounds.width !== newBounds.width || bounds.height !== newBounds.height)) {
            onResize && onResize(newBounds.width, newBounds.height);
            setBounds(newBounds);
        }
    }, [bounds.height, bounds.width, onResize]);
    const handleResize = React.useCallback((entries) => {
        // istanbul ignore next
        if (!isMountedRef.current || !Array.isArray(entries) || 0 === entries.length) {
            return;
        }
        const target = entries[0].target;
        // istanbul ignore next
        if (owningWindowRef.current)
            owningWindowRef.current.removeEventListener("beforeunload", resizeObserverCleanup);
        owningWindowRef.current = target.ownerDocument.defaultView;
        // istanbul ignore else
        if (owningWindowRef.current) {
            owningWindowRef.current.addEventListener("beforeunload", resizeObserverCleanup);
            // using requestAnimationFrame to stop the "ResizeObserver loop completed with undelivered notifications." and
            // "ResizeObserver loop limit exceeded" messages reported to window.onError
            rafRef.current = owningWindowRef.current.requestAnimationFrame(() => processResize(target));
        }
    }, [processResize]);
    React.useLayoutEffect(() => {
        if (!watchedElement) {
            return;
        }
        resizeObserver.current = new ResizeObserverPolyfill_1.ResizeObserver(handleResize);
        resizeObserver.current.observe(watchedElement);
        return () => {
            var _a;
            // istanbul ignore next
            (_a = resizeObserver.current) === null || _a === void 0 ? void 0 : _a.disconnect();
            resizeObserver.current = null;
        };
    }, [handleResize, watchedElement]);
    return [bounds.width, bounds.height];
}
exports.useLayoutResizeObserver = useLayoutResizeObserver;
/** ElementResizeObserver provides functionality similar to ReactResizeDetector when a render function is specified. This implementation properly handles
 * observing element in pop-out/child windows.
 *  @public
 */
function ElementResizeObserver({ watchedElement, render }) {
    const [width, height] = useLayoutResizeObserver(watchedElement);
    return render({ width, height });
}
exports.ElementResizeObserver = ElementResizeObserver;
/** ResizableContainerObserver is a component that provides the functionality similar to the ReactResizeDetector option that call a function when
 * the observed element is resized. This implementation properly handles observing element in pop-out/child windows. If children nodes are defined then
 * the div added by the component is considered the container whose size will be observed. If no children are provided then the component will report
 * size changes from its parent container.
 * @public
 */
function ResizableContainerObserver({ onResize, children }) {
    const containerRef = React.useRef(null); // set to non-zero when requestAnimationFrame processing is active
    const [containerElement, setContainerElement] = React.useState(null);
    const isMountedRef = React.useRef(false);
    const hasChildren = React.Children.count(children) !== 0;
    // if no children are specified then monitor size of parent element.
    React.useEffect(() => {
        // istanbul ignore next
        if (!isMountedRef.current && containerRef.current) {
            isMountedRef.current = true;
            const hasParent = !!containerRef.current.parentElement;
            setContainerElement(hasChildren && hasParent ? containerRef.current : containerRef.current.parentElement);
        }
    }, [hasChildren]);
    const processResize = React.useCallback((width, height) => {
        // istanbul ignore next
        onResize(width !== null && width !== void 0 ? width : 0, height !== null && height !== void 0 ? height : 0);
    }, [onResize]);
    useLayoutResizeObserver(containerElement, processResize);
    const style = hasChildren ? { width: "100%", height: "100%" } : { display: "none" };
    return (React.createElement("div", { ref: containerRef, className: "uicore-resizable-container", style: style }, hasChildren && children));
}
exports.ResizableContainerObserver = ResizableContainerObserver;
//# sourceMappingURL=useResizeObserver.js.map